@@ -61,14 +61,14 @@ Version:	%(wb_version_package_string)s
 Group:		Development/Libraries
 Release:	%(wb_version_package_release_string)s
 Summary:	pysvn WorkBench %(wb_version_string)s for Subversion %(svn_version_string)s
-License:	Apache Software License, Version 1.1 - Copyright Barry A. Scott (c) 2003-2005
+License:	Apache Software License, Version 1.1 - Copyright Barry A. Scott (c) 2003-2006
 Packager:	Barry A. Scott <barry@barrys-emacs.org>
 AutoReqProv:	no
 Requires:	libsvn_client-1.so.0
 %%description
 PySVN WorkBench %(wb_version_string)s for Subversion %(svn_version_string)s
 
-Copyright Barry A. Scott (c) 2003-2005
+Copyright Barry A. Scott (c) 2003-2006
 
 mailto:barry@barrys-emacs.org
 http://pysvn.tigris.org
@@ -80,9 +80,15 @@ http://pysvn.tigris.org
 %%build
 %%install
 %%post
+rm -f /usr/local/bin/workbench
 ln -f -s /usr/local/workbench/wb /usr/local/bin/workbench
 %%postun
-rm -f /usr/local/bin/workbench
+# check the arg to find out if this is an update or a delete
+# 0 - delete, 1 - update
+if [ $1 = 0 ]
+then
+    rm -f /usr/local/bin/workbench
+fi
 %%files
 %%defattr (-,root,root)
 %%attr(555,root,root) /usr/local/workbench/wb
