@@ -8,5 +8,5 @@ else
 fi
 
 export LD_LIBRARY_PATH=${SVNCPP_LIB}
-export DYLD_LIBRARY_PATH=${SVNCPP_LIB}:/sw/lib/wx-2.5/lib
+export DYLD_LIBRARY_PATH=${SVNCPP_LIB}
 pythonw wb_main.py $*
