@@ -1,6 +1,6 @@
 '''
  ====================================================================
- Copyright (c) 2003-2005 Barry A Scott.  All rights reserved.
+ Copyright (c) 2003-2006 Barry A Scott.  All rights reserved.
 
  This software is licensed as described in the file LICENSE.txt,
  which you should have received as part of this distribution.
@@ -10,6 +10,7 @@
     wb_app.py
 
 '''
+import sys
 import os
 import types
 import logging
@@ -62,6 +63,8 @@ class WbApp(wx.App):
         self.frame = None
         self.all_diff_frames = []
 
+        self.__paste_data = None
+
         self.background_thread = wb_background_thread.BackgroundThread()
         self.background_thread.start()
 
@@ -103,6 +106,17 @@ class WbApp(wx.App):
         handler.setFormatter( formatter )
         self.log.addHandler( handler )
 
+        if not self.isStdIoRedirect():
+            handler = StdoutLogHandler()
+            formatter = logging.Formatter( '%(asctime)s %(levelname)s %(message)s' )
+            handler.setFormatter( formatter )
+            self.log.addHandler( handler )
+
+            handler = StdoutLogHandler()
+            formatter = logging.Formatter( '%(asctime)s %(levelname)s %(message)s' )
+            handler.setFormatter( formatter )
+            self.trace.addHandler( handler )
+
         self.log.info( 'Work Bench starting' )
 
         self.log.debug( 'debug enabled' )
@@ -158,11 +172,24 @@ class WbApp(wx.App):
         self.progress_format = None
         self.frame.setProgress( '' )
 
+    def setPasteData( self, data ):
+        self.__paste_data = data
+
+    def clearPasteData( self ):
+        self.__paste_data = None
+
+    def hasPasteData( self ):
+        return self.__paste_data is not None
+
+    def getPasteData( self ):
+        return self.__paste_data
+
     def Diff( self, file_left, title_left, file_right, title_right ):
         diff_frame = wb_diff_frame.DiffFrame( self, self.frame,
             file_left, title_left, file_right, title_right )
         # only show if the files could be read
         if diff_frame.isOk():
+            diff_frame.showAllFolds( False )
             diff_frame.Show( True )
 
             self.all_diff_frames.append( diff_frame )
@@ -422,3 +449,15 @@ class RotatingFileHandler(logging.FileHandler):
                 self.doRollover()
         logging.FileHandler.emit(self, record)
 
+class StdoutLogHandler(logging.Handler):
+    def __init__( self ):
+        logging.Handler.__init__( self )
+
+    def emit( self, record ):
+        try:
+            msg = self.format(record) + '\n'
+
+            sys.__stdout__.write( msg )
+
+        except:
+            self.handleError(record)
