@@ -1,6 +1,6 @@
 '''
  ====================================================================
- Copyright (c) 2003-2005 Barry A Scott.  All rights reserved.
+ Copyright (c) 2003-2006 Barry A Scott.  All rights reserved.
 
  This software is licensed as described in the file LICENSE.txt,
  which you should have received as part of this distribution.
@@ -11,6 +11,7 @@
 
 '''
 import wx
+import os
 
 id_log_message_text = wx.NewId()
 id_last_log_message = wx.NewId()
@@ -94,7 +95,7 @@ class DialogBuildingBlock(wx.Dialog):
         return self.log_message_ctrl.GetValue()
 
     def getForce( self ):
-        return self.force_checkbox_ctrl.GetValue()
+        return self.force_checkbox_ctrl.GetValue() != 0
 
 class ConfirmAction(DialogBuildingBlock):
     def __init__( self, parent, title, all_filenames, force_field=False ):
@@ -102,7 +103,7 @@ class ConfirmAction(DialogBuildingBlock):
 
         self.all_filenames = all_filenames
         self.add_filename_list_field = True
-        self.add_force_field = True
+        self.add_force_field = force_field
 
         self.initControl()
 
@@ -327,7 +328,7 @@ class AddDialog(wx.Dialog):
 
     def getForce( self ):
         if self.force_ctrl is None:
-            return None
+            return False
         return self.force_ctrl.GetValue() != 0
 
 class RenameFile(wx.Dialog):
@@ -395,7 +396,7 @@ class RenameFile(wx.Dialog):
 
     def getForce( self ):
         if self.force_ctrl is None:
-            return None
+            return False
         return self.force_ctrl.GetValue() != 0
 
 class GetFilename(wx.Dialog):
@@ -446,3 +447,77 @@ class GetFilename(wx.Dialog):
 
     def getNewFilename( self ):
         return self.new_filename_ctrl.GetValue()
+
+class NewFile(wx.Dialog):
+    template_suffix = '.template'
+
+    def __init__( self, parent, template_dir ):
+        wx.Dialog.__init__( self, parent, -1, 'New File' )
+
+
+
+        self.g_sizer = wx.FlexGridSizer( 0, 2, 0, 0 )
+        self.g_sizer.AddGrowableCol( 1 )
+
+        self.newfile_border = wx.StaticBox( self, -1, 'New File' )
+        self.newfile_box = wx.StaticBoxSizer( self.newfile_border, wx.VERTICAL )
+        self.newfile_box.Add( self.g_sizer, 0, wx.EXPAND )
+
+        self.newfile_text = wx.StaticText( self, -1, 'New Filename:' )
+        self.newfile_ctrl = wx.TextCtrl( self, -1, '' )
+
+        self.g_sizer.Add( self.newfile_text, 1, wx.EXPAND|wx.NORTH|wx.ALIGN_RIGHT, 5 )
+        self.g_sizer.Add( self.newfile_ctrl, 0, wx.EXPAND|wx.ALL, 5 )
+
+        try:
+            self.template_file_list = [filename[:-len(self.template_suffix)]
+                                        for filename in os.listdir( template_dir )
+                                            if filename.lower().endswith( self.template_suffix )]
+        except EnvironmentError:
+            self.template_file_list = []
+
+        self.template_text = wx.StaticText( self, -1, 'Template:' )
+        self.template_list = wx.Choice( self, -1, choices=self.template_file_list )
+        self.template_list.SetSelection( 0 )
+
+        self.g_sizer.Add( self.template_text, 1, wx.EXPAND|wx.NORTH|wx.ALIGN_RIGHT, 5 )
+        self.g_sizer.Add( self.template_list, 0, wx.EXPAND|wx.ALL, 5 )
+
+        self.button_ok = wx.Button( self, wx.ID_OK, ' OK ' )
+        self.button_ok.SetDefault()
+        self.button_cancel = wx.Button( self, wx.ID_CANCEL, ' Cancel ' )
+
+        self.h_sizer_buttons = wx.BoxSizer( wx.HORIZONTAL )
+        self.h_sizer_buttons.Add( (150, 20), 1, wx.EXPAND )
+        self.h_sizer_buttons.Add( self.button_ok, 0, wx.EXPAND|wx.EAST, 15 )
+        self.h_sizer_buttons.Add( self.button_cancel, 0, wx.EXPAND|wx.EAST, 2 )
+
+        self.v_sizer = wx.BoxSizer( wx.VERTICAL )
+        self.v_sizer.Add( self.newfile_box, 0, wx.EXPAND|wx.ALL, 5 )
+        self.v_sizer.Add( self.h_sizer_buttons, 0, wx.EXPAND|wx.ALL, 5 )
+
+        self.SetAutoLayout( True )
+        self.SetSizer( self.v_sizer )
+        self.v_sizer.Fit( self )
+        self.Layout()
+
+        self.CentreOnParent()
+
+        wx.EVT_BUTTON( self, wx.ID_OK, self.OnOk )
+        wx.EVT_BUTTON( self, wx.ID_CANCEL, self.OnCancel )
+
+    def OnOk( self, event ):
+        self.EndModal( wx.ID_OK )
+
+    def OnCancel( self, event ):
+        self.EndModal( wx.ID_CANCEL )
+
+    def getNewFilename( self ):
+        return self.newfile_ctrl.GetValue()
+
+    def getTemplateFilename( self ):
+        index = self.template_list.GetCurrentSelection()
+        if len(self.template_file_list) > index:
+            return self.template_file_list[ index ] + self.template_suffix
+        else:
+            return None
