@@ -1,6 +1,6 @@
 '''
  ====================================================================
- Copyright (c) 2003-2005 Barry A Scott.  All rights reserved.
+ Copyright (c) 2003-2006 Barry A Scott.  All rights reserved.
 
  This software is licensed as described in the file LICENSE.txt,
  which you should have received as part of this distribution.
@@ -18,6 +18,7 @@ import logging
 
 import wb_tree_panel
 import wb_list_panel
+import wb_list_panel_common
 import wb_ids
 import wb_exceptions
 import wb_version
@@ -27,6 +28,8 @@ import wb_source_control_providers
 import wb_platform_specific
 import wb_bookmarks_dialogs
 
+import wb_config
+
 # point size and face need to choosen for platform
 if wx.Platform == '__WXMSW__':
     face = 'Courier New'
@@ -61,6 +64,10 @@ class WbFrame(wx.Frame):
                 wx.DEFAULT_FRAME_STYLE|extra_style )
 
         self.menu_edit = wx.Menu()
+        self.menu_edit.Append( wb_ids.id_SP_EditCopy, "&Copy", "Copy Files" )
+        self.menu_edit.Append( wb_ids.id_SP_EditCut, "&Cut", "Cut Files" )
+        self.menu_edit.Append( wb_ids.id_SP_EditPaste, "&Paste", "Paste Files" )
+        self.menu_edit.AppendSeparator()
         self.menu_edit.Append( wb_ids.id_ClearLog, "&Clear log", "Clear the log window" )
 
         if wx.Platform != '__WXMAC__':
@@ -72,17 +79,17 @@ class WbFrame(wx.Frame):
         self.menu_file.Append( wx.ID_EXIT, "E&xit", "Exit the application" )
 
         self.menu_actions = wx.Menu()
-        self.menu_actions.Append(  wb_ids.id_Project_Add, 'Project Add', 'Project Add' )
-        self.menu_actions.AppendSeparator()
         self.menu_actions.Append(  wb_ids.id_Command_Shell, '&Command Shell', 'Command Shell' )
         self.menu_actions.Append(  wb_ids.id_File_Browser, '&File Browser', 'File Browser' )
         self.menu_actions.AppendSeparator()
         self.menu_actions.Append(  wb_ids.id_File_Edit, 'Edit', 'Edit' )
-        if sys.platform in ['win32','darwin']:
+        if wx.Platform in ['__WXMSW__','__WXMAC__']:
             self.menu_actions.Append(  wb_ids.id_Shell_Open, 'Open', 'Open' )
         self.menu_actions.AppendSeparator()
         self.menu_actions.Append(  wb_ids.id_SP_DiffWorkBase, 'Diff WC vs. BASE...', 'Diff WC vs. BASE...' )
         self.menu_actions.Append(  wb_ids.id_SP_DiffWorkHead, 'Diff WC vs. HEAD...', 'Diff WC vs. HEAD...' )
+        self.menu_actions.Append(  wb_ids.id_SP_DiffWorkBranchOriginBase, 'Diff WC vs. branch origin BASE...', 'Diff WC vs. branch origin BASE...' )
+        self.menu_actions.Append(  wb_ids.id_SP_DiffWorkBranchOriginHead, 'Diff WC vs. branch origin HEAD...', 'Diff WC vs. branch origin HEAD...' )
 
         self.menu_actions_conflict = wx.Menu()
         self.menu_actions.AppendMenu( wb_ids.id_SP_ConflictMenu, 'Conflict', self.menu_actions_conflict )
@@ -94,7 +101,7 @@ class WbFrame(wx.Frame):
 
         self.menu_actions.AppendSeparator()
         self.menu_actions.Append( wb_ids.id_SP_Annotate, 'Annotate...', 'Annotate...' )
-        self.menu_actions.Append( wb_ids.id_SP_History, 'History...', 'History...' )
+        self.menu_actions.Append( wb_ids.id_SP_History, 'Log history...', 'Log history...' )
         self.menu_actions.Append( wb_ids.id_SP_Info, 'Information...', 'Information...' )
         self.menu_actions.Append( wb_ids.id_SP_Properties, 'Properties...', 'Properties...' )
         self.menu_actions.AppendSeparator()
@@ -106,15 +113,24 @@ class WbFrame(wx.Frame):
         self.menu_actions.Append( wb_ids.id_SP_Lock, 'Lock...', 'Lock...' )
         self.menu_actions.Append( wb_ids.id_SP_Unlock, 'Unlock...', 'Unlock...' )
         self.menu_actions.AppendSeparator()
+        self.menu_actions.Append( wb_ids.id_SP_NewFile, 'New File...', 'New File...' )
         self.menu_actions.Append( wb_ids.id_SP_Mkdir, 'Make directory...', 'Make directory...' )
         self.menu_actions.Append( wb_ids.id_SP_Add, 'Add', 'Add' )
-        self.menu_actions.Append( wb_ids.id_SP_Rename, 'Rename', 'Rename' )
+        self.menu_actions.Append( wb_ids.id_SP_Rename, 'Rename...', 'Rename' )
         self.menu_actions.AppendSeparator()
-        self.menu_actions.Append( wb_ids.id_SP_Delete, 'Delete', 'Delete' )
-        self.menu_actions.Append( wb_ids.id_SP_Revert, 'Revert', 'Revert' )
+        self.menu_actions.Append( wb_ids.id_SP_Delete, 'Delete...', 'Delete' )
+        self.menu_actions.Append( wb_ids.id_SP_Revert, 'Revert...', 'Revert' )
         self.menu_actions.AppendSeparator()
         self.menu_actions.Append( wb_ids.id_SP_Cleanup, 'Clean up', 'Clean up working copy' )
 
+        self.menu_reports = wx.Menu()
+        self.menu_reports.Append( wb_ids.id_SP_Report_LocksWc, 'Working copy Locks...', 'Locks held in Working Copy' )
+        self.menu_reports.Append( wb_ids.id_SP_Report_LocksRepos, 'Repository Locks...', 'Locks held in Repository' )
+        self.menu_reports.AppendSeparator()
+        self.menu_reports.Append( wb_ids.id_SP_Checkin, 'Changes...', 'Changes available for checkin' )
+        self.menu_reports.Append( wb_ids.id_SP_Report_Updates, 'Updates...', 'Updates available in the Repository' )
+        self.menu_reports.Append( wb_ids.id_SP_Report_BranchChanges, 'Branch changes...', 'Files changed in this branch' )
+
         self.menu_view = wx.Menu()
         self.menu_view.AppendCheckItem( wb_ids.id_View_ControlledFiles, "Show &Controlled files", "Show Controlled files" )
         self.menu_view.AppendCheckItem( wb_ids.id_View_UncontrolledFiles, "Show &Uncontrolled files", "Show Uncontrolled files" )
@@ -125,15 +141,23 @@ class WbFrame(wx.Frame):
         self.menu_view.Append( wb_ids.id_View_Refresh, "&Refresh\tF5", "Refresh display" )
         self.menu_view.AppendCheckItem( wb_ids.id_View_AutoRefresh, "&Automatic Refresh", "Automatic refresh" )
 
-        self.bookmark_id_map = {}
+        self.all_bookmark_ids = {}
+        self.all_bookmark_top_level_menu_ids = []
+        self.all_bookmark_folders = {}
 
         self.menu_bookmarks = wx.Menu()
-        self.menu_bookmarks.Append(  wb_ids.id_Bookmark_Add, 'Add', 'Add Bookmark' )
-        self.menu_bookmarks.Append(  wb_ids.id_Bookmark_Manage, 'Manage...', 'Manage Bookmarks' )
+        self.menu_bookmarks.Append( wb_ids.id_Bookmark_Add, 'Add', 'Add Bookmark' )
+        self.menu_bookmarks.Append( wb_ids.id_Bookmark_Manage, 'Manage...', 'Manage Bookmarks' )
         self.menu_bookmarks.AppendSeparator()
 
         self.__bookmarkMenuReorder()
 
+        self.menu_project = wx.Menu()
+        self.menu_project.Append( wb_ids.id_Project_Add, 'Add...', 'Project Add' )
+        self.menu_project.Append( wb_ids.id_Project_Update, 'Update...', 'Update Project' )
+        self.menu_project.AppendSeparator()
+        self.menu_project.Append( wb_ids.id_Project_Delete, 'Delete...', 'Delete Project' )
+
         self.menu_help = wx.Menu()
         self.menu_help.Append( wx.ID_ABOUT, "&About...", "About the application" )
 
@@ -143,13 +167,85 @@ class WbFrame(wx.Frame):
         self.menu_bar.Append( self.menu_edit, "&Edit" )
         self.menu_bar.Append( self.menu_view, "&View" )
         self.menu_bar.Append( self.menu_actions, "&Actions" )
+        self.menu_bar.Append( self.menu_reports, "&Reports" )
         self.menu_bar.Append( self.menu_bookmarks, "&Bookmarks" )
+        self.menu_bar.Append( self.menu_project, "&Project" )
         self.menu_bar.Append( self.menu_help, "&Help" )
 
         self.SetMenuBar( self.menu_bar )
 
         # Set the application icon
-        self.SetIcon( wb_images.getAppIconIcon() )
+        self.SetIcon( wb_images.getIcon( 'wb.png') )
+
+        # Add tool bar
+        t = self.CreateToolBar( name="main",
+                                style=wx.TB_HORIZONTAL ) # | wx.NO_BORDER | wx.TB_TEXT )
+
+        bitmap_size = (32,32)
+        t.SetToolBitmapSize( bitmap_size )
+        t.AddSimpleTool( wb_ids.id_SP_EditCut,
+            wb_images.getBitmap( 'toolbar_images/editcut.png', bitmap_size ),
+            'Cut Files and Folders', 'Cut Files and Folders' )
+        t.AddSimpleTool( wb_ids.id_SP_EditCopy,
+            wb_images.getBitmap( 'toolbar_images/editcopy.png', bitmap_size ),
+            'Copy Files and Folders', 'Copy Files and Folders' )
+        t.AddSimpleTool( wb_ids.id_SP_EditPaste,
+            wb_images.getBitmap( 'toolbar_images/editpaste.png', bitmap_size ),
+            'Paste Files and Folders', 'Paste Files and Folders' )
+        t.AddSeparator()
+        t.AddSimpleTool( wb_ids.id_Command_Shell,
+            wb_images.getBitmap( 'toolbar_images/terminal.png', bitmap_size ),
+            'Command Shell', 'Start new command shell' )
+        t.AddSimpleTool( wb_ids.id_File_Browser,
+            wb_images.getBitmap( 'toolbar_images/file_browser.png', bitmap_size ),
+            'File Browser', 'File Browser' )
+        t.AddSeparator()
+        t.AddSimpleTool( wb_ids.id_File_Edit,
+            wb_images.getBitmap( 'toolbar_images/edit.png', bitmap_size ),
+            'Edit File', 'Edit File' )
+        if wx.Platform in ['__WXMSW__','__WXMAC__']:
+            t.AddSimpleTool( wb_ids.id_Shell_Open,
+                wb_images.getBitmap( 'toolbar_images/open.png', bitmap_size ),
+                'Open File', 'Open File' )
+        t.AddSeparator()
+        t.AddSimpleTool( wb_ids.id_SP_DiffWorkBase,
+            wb_images.getBitmap( 'toolbar_images/diff.png', bitmap_size ),
+            'Diff changes against base', 'Diff changes against base' )
+        t.AddSimpleTool( wb_ids.id_SP_History,
+            wb_images.getBitmap( 'toolbar_images/history.png', bitmap_size ),
+            'Show History log', 'Show History log' )
+        t.AddSimpleTool( wb_ids.id_SP_Info,
+            wb_images.getBitmap( 'toolbar_images/info.png', bitmap_size ),
+            'File Information', 'File Information' )
+        t.AddSimpleTool( wb_ids.id_SP_Properties,
+            wb_images.getBitmap( 'toolbar_images/property.png', bitmap_size ),
+            'File Properties', 'File Properties' )
+        t.AddSeparator()
+        t.AddSimpleTool( wb_ids.id_SP_Add,
+            wb_images.getBitmap( 'toolbar_images/add.png', bitmap_size ),
+            'Add Files and Folders', 'Add Files and Folders' )
+        t.AddSimpleTool( wb_ids.id_SP_Delete,
+            wb_images.getBitmap( 'toolbar_images/delete.png', bitmap_size ),
+            'Delete selected Files and Folders', 'Delete selected Files and Folders' )
+        t.AddSimpleTool( wb_ids.id_SP_Revert,
+            wb_images.getBitmap( 'toolbar_images/revert.png', bitmap_size ),
+            'Revert selected Files and Folders', 'Revert selected Files and Folders' )
+        t.AddSeparator()
+        t.AddSimpleTool( wb_ids.id_SP_Lock,
+            wb_images.getBitmap( 'toolbar_images/lock.png', bitmap_size ),
+            'Lock File', 'Lock File' )
+        t.AddSimpleTool( wb_ids.id_SP_Unlock,
+            wb_images.getBitmap( 'toolbar_images/unlock.png', bitmap_size ),
+            'Unlock File', 'Unlock File' )
+        t.AddSeparator()
+        t.AddSimpleTool( wb_ids.id_SP_Checkin,
+            wb_images.getBitmap( 'toolbar_images/checkin.png', bitmap_size ),
+            'Checkin changes', 'Checkin changes' )
+        t.AddSimpleTool( wb_ids.id_SP_Update,
+            wb_images.getBitmap( 'toolbar_images/update.png', bitmap_size ),
+            'Update working copy', 'Update working copy' )
+
+        t.Realize()
 
         # Add the status bar
         s = self.CreateStatusBar()
@@ -185,8 +281,6 @@ class WbFrame(wx.Frame):
         self.v_split.SplitVertically( self.tree_panel, self.list_panel, v_sash_pos )
         self.h_split.SplitHorizontally( self.v_split, self.log_panel, h_sash_pos )
 
-        #wx.EVT_MENU_OPEN( self, self.OnMenuOpen )
-
         # for some unknown reason MENU events get blocked by tree and list controls
         for event_source in [self, self.tree_panel.tree_ctrl, self.list_panel.list_ctrl]:
             # Set up the event handlers
@@ -212,6 +306,14 @@ class WbFrame(wx.Frame):
             wx.EVT_MENU( event_source, wb_ids.id_View_Recursive, try_wrapper( self.OnToggleViewRecursive ) )
             wx.EVT_UPDATE_UI( event_source, wb_ids.id_View_Recursive, try_wrapper( self.OnUpdateViewRecursive ) )
 
+            wx.EVT_MENU( event_source, wb_ids.id_SP_EditCopy, self.app.eventWrapper( self.OnSpEditCopy ) )
+            wx.EVT_UPDATE_UI( event_source, wb_ids.id_SP_EditCopy, self.app.eventWrapper( self.OnUpdateUiSpEditCopy ) )
+            wx.EVT_MENU( event_source, wb_ids.id_SP_EditCut, self.app.eventWrapper( self.OnSpEditCut ) )
+            wx.EVT_UPDATE_UI( event_source, wb_ids.id_SP_EditCut, self.app.eventWrapper( self.OnUpdateUiSpEditCut ) )
+            wx.EVT_MENU( event_source, wb_ids.id_SP_EditPaste, self.app.eventWrapper( self.OnSpEditPaste ) )
+            wx.EVT_UPDATE_UI( event_source, wb_ids.id_SP_EditPaste, self.app.eventWrapper( self.OnUpdateUiSpEditPaste ) )
+
+
         wx.EVT_MENU( self, wb_ids.id_File_Edit, try_wrapper( self.OnFileEdit ) )
         wx.EVT_UPDATE_UI( self, wb_ids.id_File_Edit, try_wrapper( self.OnUpdateUiFileEdit ) )
         wx.EVT_MENU( self, wb_ids.id_Shell_Open, try_wrapper( self.OnShellOpen ) )
@@ -244,12 +346,18 @@ class WbFrame(wx.Frame):
         wx.EVT_UPDATE_UI( self, wb_ids.id_SP_DiffWorkBase, self.app.eventWrapper( self.OnUpdateUiSpDiffWorkBase ) )
         wx.EVT_MENU( self, wb_ids.id_SP_DiffWorkHead, self.app.eventWrapper( self.OnSpDiffWorkHead ) )
         wx.EVT_UPDATE_UI( self, wb_ids.id_SP_DiffWorkHead, self.app.eventWrapper( self.OnUpdateUiSpDiffWorkHead ) )
+        wx.EVT_MENU( self, wb_ids.id_SP_DiffWorkBranchOriginBase, self.app.eventWrapper( self.OnSpDiffWorkBranchOriginBase ) )
+        wx.EVT_UPDATE_UI( self, wb_ids.id_SP_DiffWorkBranchOriginBase, self.app.eventWrapper( self.OnUpdateUiSpDiffWorkBranchOriginBase ) )
+        wx.EVT_MENU( self, wb_ids.id_SP_DiffWorkBranchOriginHead, self.app.eventWrapper( self.OnSpDiffWorkBranchOriginHead ) )
+        wx.EVT_UPDATE_UI( self, wb_ids.id_SP_DiffWorkBranchOriginHead, self.app.eventWrapper( self.OnUpdateUiSpDiffWorkBranchOriginHead ) )
         wx.EVT_MENU( self, wb_ids.id_SP_History, self.app.eventWrapper( self.OnSpHistory ) )
         wx.EVT_UPDATE_UI( self, wb_ids.id_SP_History, self.app.eventWrapper( self.OnUpdateUiSpHistory ) )
         wx.EVT_MENU( self, wb_ids.id_SP_Info, self.app.eventWrapper( self.OnSpInfo ) )
         wx.EVT_UPDATE_UI( self, wb_ids.id_SP_Info, self.app.eventWrapper( self.OnUpdateUiSpInfo ) )
         wx.EVT_MENU( self, wb_ids.id_SP_Lock, self.app.eventWrapper( self.OnSpLock ) )
         wx.EVT_UPDATE_UI( self, wb_ids.id_SP_Lock, self.app.eventWrapper( self.OnUpdateUiSpLock ) )
+        wx.EVT_MENU( self, wb_ids.id_SP_NewFile, self.app.eventWrapper( self.OnSpNewFile ) )
+        wx.EVT_UPDATE_UI( self, wb_ids.id_SP_NewFile, self.app.eventWrapper( self.OnUpdateUiSpNewFile ) )
         wx.EVT_MENU( self, wb_ids.id_SP_Mkdir, self.app.eventWrapper( self.OnSpMkdir ) )
         wx.EVT_UPDATE_UI( self, wb_ids.id_SP_Mkdir, self.app.eventWrapper( self.OnUpdateUiSpMkdir ) )
         wx.EVT_MENU( self, wb_ids.id_SP_Properties, self.app.eventWrapper( self.OnSpProperties ) )
@@ -265,7 +373,20 @@ class WbFrame(wx.Frame):
         wx.EVT_MENU( self, wb_ids.id_SP_Update, self.app.eventWrapper( self.OnSpUpdate ) )
         wx.EVT_UPDATE_UI( self, wb_ids.id_SP_Update, self.app.eventWrapper( self.OnUpdateUiSpUpdate ) )
 
-        wx.EVT_MENU( self, wb_ids.id_Project_Add, try_wrapper( self.tree_panel.OnProjectAdd ) )
+        wx.EVT_MENU( self, wb_ids.id_SP_Report_Updates, self.app.eventWrapper( self.OnSpReportUpdates ) )
+        wx.EVT_UPDATE_UI( self, wb_ids.id_SP_Report_Updates, self.app.eventWrapper( self.OnUpdateUiSpReportUpdates ) )
+        wx.EVT_MENU( self, wb_ids.id_SP_Report_LocksWc, self.app.eventWrapper( self.OnSpReportLocksWc ) )
+        wx.EVT_UPDATE_UI( self, wb_ids.id_SP_Report_LocksWc, self.app.eventWrapper( self.OnUpdateUiSpReportLocksWc ) )
+        wx.EVT_MENU( self, wb_ids.id_SP_Report_LocksRepos, self.app.eventWrapper( self.OnSpReportLocksRepos ) )
+        wx.EVT_UPDATE_UI( self, wb_ids.id_SP_Report_LocksRepos, self.app.eventWrapper( self.OnUpdateUiSpReportLocksRepos ) )
+        wx.EVT_MENU( self, wb_ids.id_SP_Report_BranchChanges, self.app.eventWrapper( self.OnSpReportBranchChanges ) )
+        wx.EVT_UPDATE_UI( self, wb_ids.id_SP_Report_BranchChanges, self.app.eventWrapper( self.OnUpdateUiSpReportBranchChanges ) )
+
+        wx.EVT_MENU( self, wb_ids.id_Project_Add, try_wrapper( self.app.eventWrapper( self.tree_panel.OnProjectAdd ) ) )
+        wx.EVT_MENU( self, wb_ids.id_Project_Update, try_wrapper( self.app.eventWrapper( self.tree_panel.OnProjectUpdate ) ) )
+        wx.EVT_UPDATE_UI( self, wb_ids.id_Project_Update, try_wrapper( self.app.eventWrapper( self.OnUpdateUiProjectUpdateOrDelete ) ) )
+        wx.EVT_MENU( self, wb_ids.id_Project_Delete, try_wrapper( self.app.eventWrapper( self.tree_panel.OnProjectDelete ) ) )
+        wx.EVT_UPDATE_UI( self, wb_ids.id_Project_Delete, try_wrapper( self.app.eventWrapper( self.OnUpdateUiProjectUpdateOrDelete ) ) )
 
         wx.EVT_MENU( self, wb_ids.id_Bookmark_Add, try_wrapper( self.OnBookmarkAdd ) )
         wx.EVT_MENU( self, wb_ids.id_Bookmark_Manage, try_wrapper( self.OnBookmarkManage ) )
@@ -294,12 +415,14 @@ class WbFrame(wx.Frame):
         self.list_panel.list_ctrl.SetFocus()
 
     def setEventHandler( self, handler ):
-        #print 'setEventHandler %r' % handler
-        self.event_handler = handler
-        self.clearUpdateUiState()
+        self.app.log.debug( 'setEventHandler from %r' % self.event_handler )
+        self.app.log.debug( 'setEventHandler   to %r' % handler )
+        if self.event_handler is not handler:
+            self.event_handler = handler
+            self.clearUpdateUiState()
 
     def clearEventHandler( self ):
-        #print 'clearEventHandler was %r' % self.event_handler
+        self.app.log.debug( 'clearEventHandler from %r to None' % self.event_handler )
         self.event_handler = None
         self.clearUpdateUiState()
 
@@ -330,12 +453,13 @@ class WbFrame(wx.Frame):
 
     # Handler for the About menu command
     def OnCmdAbout(self, event):
-        strMessage =    ('Work Bench version: %d.%d.%d-%d\n\n' %
+        strMessage =    ('Work Bench version: %d.%d.%d-%d\n' %
                     (wb_version.major, wb_version.minor,
                      wb_version.patch, wb_version.build) +
-                wb_source_control_providers.getProviderAboutStrings() +
-                'WxPython %d.%d.%d.%d %s' % wx.VERSION +
-                '\nCopyright Barry Scott (c) 2003-2005. All rights reserved'
+                '\n' + wb_source_control_providers.getProviderAboutStrings() +
+                '\nWxPython %d.%d.%d.%d %s' % wx.VERSION +
+                '\nPython %d.%d.%d %s %d\n' % sys.version_info +
+                '\nCopyright Barry Scott (c) 2003-2006. All rights reserved'
                 )
         wx.LogMessage( strMessage )
 
@@ -458,20 +582,36 @@ class WbFrame(wx.Frame):
         self.tree_panel.selectTreeNode( filename )
 
     #------------------------------------------------------------------------
-    def __addBookmarkMenuItem( self, menu_name, pi ):
+    def __addBookmarkMenuItem( self, pi ):
         bm_id = wx.NewId()
+        self.all_bookmark_ids[ bm_id ] = pi
 
-        self.bookmark_id_map[ bm_id ] = pi
+        if pi.menu_folder == '':
+            menu = self.menu_bookmarks
+            self.all_bookmark_top_level_menu_ids.append( bm_id )
+        else:
+            if pi.menu_folder not in self.all_bookmark_folders:
+                menu_id = wx.NewId()
+                self.all_bookmark_top_level_menu_ids.append( menu_id )
 
-        self.menu_bookmarks.Append( bm_id, menu_name, pi.wc_path )
+                menu = wx.Menu()
+
+                self.all_bookmark_folders[ pi.menu_folder ] = menu
+                self.menu_bookmarks.AppendMenu( menu_id, pi.menu_folder, menu )
+            else:
+                menu = self.all_bookmark_folders[ pi.menu_folder ]
+
+        menu.Append( bm_id, pi.menu_name, pi.wc_path )
         try_wrapper = wb_exceptions.TryWrapperFactory( self.app.log )
         wx.EVT_MENU( self, bm_id, try_wrapper( self.OnBookmarkGoto ) )
 
     def __bookmarkMenuReorder( self ):
-        for menu_id in self.bookmark_id_map.keys():
+        for menu_id in self.all_bookmark_top_level_menu_ids:
             self.menu_bookmarks.Delete( menu_id )
 
-        self.bookmark_id_map = {}
+        self.all_bookmark_ids = {}
+        self.all_bookmark_folders = {}
+        self.all_bookmark_top_level_menu_ids = []
 
         bm_prefs = self.app.prefs.getBookmarks()
         bookmark_list = []
@@ -479,13 +619,18 @@ class WbFrame(wx.Frame):
             if bm_name == 'last position':
                 continue
 
-            pi = bm_prefs.getBookmark( bm_name )
-            bookmark_list.append( (bm_prefs.getMenuName( pi.wc_path ), pi) )
+            bookmark_list.append( bm_prefs.getBookmark( bm_name ) )
 
-        bookmark_list.sort( lambda a, b: cmp( a[0].lower(), b[0].lower() ) )
+        def sortFunction( a, b ):
+            rc = cmp( a.menu_folder.lower(), b.menu_folder.lower() )
+            if rc == 0:
+                rc = cmp( a.menu_name.lower(), b.menu_name.lower() )
+            return rc
 
-        for menu_name, pi in bookmark_list:
-            self.__addBookmarkMenuItem( menu_name, pi )
+        bookmark_list.sort( sortFunction )
+
+        for pi in bookmark_list:
+            self.__addBookmarkMenuItem( pi )
 
     def OnBookmarkAdd( self, event ):
         pi = self.tree_panel.getSelectionProjectInfo()
@@ -495,15 +640,15 @@ class WbFrame(wx.Frame):
  
         if not bm_prefs.hasBookmark( pi.url ):
             print 'Adding bookmark to %s' % pi.wc_path
-            bm_prefs.addBookmark( pi.wc_path, pi )
+            bm_prefs.addBookmark( pi )
 
-            self.__addBookmarkMenuItem( bm_prefs.getMenuName( pi.wc_path ), pi )
+            self.__addBookmarkMenuItem( pi )
             self.__bookmarkMenuReorder()
             self.app.savePreferences()
 
     def OnBookmarkManage( self, event ):
         bookmarks = self.app.prefs.getBookmarks()
-        dialog = wb_bookmarks_dialogs.BookmarkManageDialog( self, bookmarks )
+        dialog = wb_bookmarks_dialogs.BookmarkManageDialog( self, self.app, bookmarks )
         rc = dialog.ShowModal()
         if rc != wx.ID_OK:
             return
@@ -513,7 +658,7 @@ class WbFrame(wx.Frame):
         self.__bookmarkMenuReorder()
 
     def OnBookmarkGoto( self, event ):
-        self.tree_panel.gotoBookmark( self.bookmark_id_map[event.GetId()].wc_path )
+        self.tree_panel.gotoBookmark( self.all_bookmark_ids[event.GetId()].wc_path )
 
     #------------------------------------------------------------------------
     def OnCmdClearLog( self, event ):
@@ -555,6 +700,28 @@ class WbFrame(wx.Frame):
         event.Enable( self.ui_state_tree.file_exists )
 
 
+    def OnSpEditCopy( self, event ):
+        return self.Sp_Dispatch( 'OnSpEditCopy' )
+
+    def OnUpdateUiSpEditCopy( self, event ):
+        self.getUpdateUiState()
+        event.Enable( self.ui_state_focus.file_exists )
+
+    def OnSpEditCut( self, event ):
+        return self.Sp_Dispatch( 'OnSpEditCut' )
+
+    def OnUpdateUiSpEditCut( self, event ):
+        self.getUpdateUiState()
+        event.Enable( self.ui_state_focus.file_exists )
+
+    def OnSpEditPaste( self, event ):
+        return self.Sp_Dispatch( 'OnSpEditPaste' )
+
+    def OnUpdateUiSpEditPaste( self, event ):
+        self.getUpdateUiState()
+        event.Enable( self.app.hasPasteData() )
+
+    #----------------------------------------
     def OnSpAdd( self, event ):
         return self.Sp_Dispatch( 'OnSpAdd' )
 
@@ -567,16 +734,23 @@ class WbFrame(wx.Frame):
 
     def OnUpdateUiSpAnnotate( self, event ):
         self.getUpdateUiState()
-        event.Enable( self.ui_state_tree.versioned )
+        if self.ui_state_focus.need_checkout:
+            event.Enable( False )
+        else:
+            event.Enable( self.ui_state_tree.versioned )
 
     def OnSpCheckin( self, event ):
         return self.Sp_Dispatch( 'OnSpCheckin' )
 
     def OnUpdateUiSpCheckin( self, event ):
         self.getUpdateUiState()
-
-        event.Enable( self.ui_state_focus.need_checkin
-                or (self.event_handler is not None and self.event_handler.isTreeHandler()) )
+        if self.ui_state_focus.need_checkout:
+            event.Enable( False )
+        else:
+            event.Enable( self.ui_state_focus.need_checkin
+                or (self.ui_state_focus.versioned
+                    and self.event_handler is not None
+                    and self.event_handler.isTreeHandler()) )
 
     def OnSpCheckout( self, event ):
         self.clearUpdateUiState()
@@ -635,23 +809,52 @@ class WbFrame(wx.Frame):
     def OnSpDiffWorkHead( self, event ):
         return self.Sp_Dispatch( 'OnSpDiffWorkHead' )
 
+    def OnSpDiffWorkBranchOriginBase( self, event ):
+        return self.Sp_Dispatch( 'OnSpDiffWorkBranchOriginBase' )
+
+    def OnSpDiffWorkBranchOriginHead( self, event ):
+        return self.Sp_Dispatch( 'OnSpDiffWorkBranchOriginHead' )
+
     def OnUpdateUiSpDiffWorkHead( self, event ):
         self.getUpdateUiState()
-        event.Enable( self.ui_state_list.versioned and not self.ui_state_list.new_versioned )
+        if self.ui_state_focus.need_checkout:
+            event.Enable( False )
+        else:
+            event.Enable( self.ui_state_list.versioned and not self.ui_state_list.new_versioned )
+
+    def OnUpdateUiSpDiffWorkBranchOriginBase( self, event ):
+        self.getUpdateUiState()
+        if self.ui_state_focus.need_checkout:
+            event.Enable( False )
+        else:
+            event.Enable( self.ui_state_list.versioned and not self.ui_state_list.new_versioned )
+
+    def OnUpdateUiSpDiffWorkBranchOriginHead( self, event ):
+        self.getUpdateUiState()
+        if self.ui_state_focus.need_checkout:
+            event.Enable( False )
+        else:
+            event.Enable( self.ui_state_list.versioned and not self.ui_state_list.new_versioned )
 
     def OnSpHistory( self, event ):
         return self.Sp_Dispatch( 'OnSpHistory' )
 
     def OnUpdateUiSpHistory( self, event ):
         self.getUpdateUiState()
-        event.Enable( self.ui_state_focus.versioned and not self.ui_state_focus.new_versioned )
+        if self.ui_state_focus.need_checkout:
+            event.Enable( False )
+        else:
+            event.Enable( self.ui_state_focus.versioned and not self.ui_state_focus.new_versioned )
 
     def OnSpInfo( self, event ):
         return self.Sp_Dispatch( 'OnSpInfo' )
 
     def OnUpdateUiSpInfo( self, event ):
         self.getUpdateUiState()
-        event.Enable( self.ui_state_focus.versioned )
+        if self.ui_state_focus.need_checkout:
+            event.Enable( False )
+        else:
+            event.Enable( self.ui_state_focus.versioned )
 
     def OnSpLock( self, event ):
         return self.Sp_Dispatch( 'OnSpLock' )
@@ -667,12 +870,22 @@ class WbFrame(wx.Frame):
         self.getUpdateUiState()
         event.Enable( self.ui_state_tree.file_exists )
 
+    def OnSpNewFile( self, event ):
+        return self.Sp_Dispatch( 'OnSpNewFile' )
+
+    def OnUpdateUiSpNewFile( self, event ):
+        self.getUpdateUiState()
+        event.Enable( self.ui_state_tree.file_exists )
+
     def OnSpProperties( self, event ):
         return self.Sp_Dispatch( 'OnSpProperties' )
 
     def OnUpdateUiSpProperties( self, event ):
         self.getUpdateUiState()
-        event.Enable( self.ui_state_focus.versioned )
+        if self.ui_state_focus.need_checkout:
+            event.Enable( False )
+        else:
+            event.Enable( self.ui_state_focus.versioned )
 
     def OnSpRename( self, event ):
         return self.Sp_Dispatch( 'OnSpRename' )
@@ -681,6 +894,34 @@ class WbFrame(wx.Frame):
         self.getUpdateUiState()
         event.Enable( self.ui_state_focus.file_exists )
 
+    def OnSpReportUpdates( self, event ):
+        return self.Sp_Dispatch( 'OnReportUpdates' )
+
+    def OnUpdateUiSpReportUpdates( self, event ):
+        self.getUpdateUiState()
+        event.Enable( self.ui_state_focus.versioned )
+
+    def OnSpReportLocksWc( self, event ):
+        return self.Sp_Dispatch( 'OnReportLocksWc' )
+
+    def OnUpdateUiSpReportLocksWc( self, event ):
+        self.getUpdateUiState()
+        event.Enable( self.ui_state_focus.versioned )
+
+    def OnSpReportBranchChanges( self, event ):
+        return self.Sp_Dispatch( 'OnReportBranchChanges' )
+
+    def OnUpdateUiSpReportBranchChanges( self, event ):
+        self.getUpdateUiState()
+        event.Enable( self.ui_state_focus.versioned )
+
+    def OnSpReportLocksRepos( self, event ):
+        return self.Sp_Dispatch( 'OnReportLocksRepos' )
+
+    def OnUpdateUiSpReportLocksRepos( self, event ):
+        self.getUpdateUiState()
+        event.Enable( self.ui_state_focus.versioned )
+
     def OnSpResolved( self, event ):
         return self.Sp_Dispatch( 'OnSpResolved' )
 
@@ -693,7 +934,13 @@ class WbFrame(wx.Frame):
 
     def OnUpdateUiSpRevert( self, event ):
         self.getUpdateUiState()
-        event.Enable( self.ui_state_focus.need_checkin or not self.ui_state_focus.file_exists )
+        if self.ui_state_focus.need_checkout:
+            event.Enable( False )
+        else:
+            event.Enable( self.ui_state_focus.need_checkin
+                or self.ui_state_focus.conflict
+                or (not self.ui_state_focus.file_exists
+                    and self.ui_state_focus.versioned) )
 
     def OnSpUnlock( self, event ):
         return self.Sp_Dispatch( 'OnSpUnlock' )
@@ -708,46 +955,70 @@ class WbFrame(wx.Frame):
     def OnUpdateUiSpUpdate( self, event ):
         self.getUpdateUiState()
 
-        if self.ui_state_focus.is_project_parent:
+        if self.ui_state_focus.need_checkout:
+            event.Enable( False )
+
+        elif self.ui_state_focus.is_project_parent:
             event.Enable( self.ui_state_focus.versioned and self.ui_state_focus.file_exists )
+
         else:
             event.Enable( self.ui_state_focus.versioned )
 
+    def OnUpdateUiProjectUpdateOrDelete( self, event ):
+        handler = self.tree_panel.getSelectionProjectHandler()
+        if handler and handler.isProjectParent():
+            event.Enable( True )
+        else:
+            event.Enable( False )
+
+    #----------------------------------------
     def Sp_Dispatch( self, sp_func_name ):
         self.clearUpdateUiState()
 
         if self.event_handler is None:
-            print 'No event_handler cannot call',sp_func_name
+            print 'No event_handler, cannot call',sp_func_name
             return None
 
-        return getattr( self.event_handler, sp_func_name, self.returnNone )()    
+        fn = getattr( self.event_handler, sp_func_name, None )
+        if fn is None:
+            print 'Not implemented:',sp_func_name
+            return None
+        else:
+            return fn()
 
     def getUpdateUiState( self ):
         if self.ui_state_tree is None:
-            #print 'getUpdateUiState() tree'
+            if wb_config.debug_selection: print 'Z: getUpdateUiState() tree'
             self.ui_state_tree = self.tree_panel.getUpdateUiState()
             if self.ui_state_tree is None:
                 self.ui_state_tree = wb_tree_panel.TreeState()
+            if wb_config.debug_selection: print 'Z: getUpdateUiState() tree need_checkout',self.ui_state_tree.need_checkout
 
         if self.ui_state_list is None:
-            #print 'getUpdateUiState() list'
+            if wb_config.debug_selection: print 'Z: getUpdateUiState() list'
             self.ui_state_list = self.list_panel.getUpdateUiState()
             if self.ui_state_list is None:
-                self.ui_state_list = wb_list_panel.ListItemState()
-        if self.event_handler is None:
-            self.ui_state_focus = self.ui_state_tree
-        elif self.event_handler.isTreeHandler():
-            self.ui_state_focus = self.ui_state_tree
-        else:
-            self.ui_state_focus = self.ui_state_list
+                self.ui_state_list = wb_list_panel_common.ListItemState()
+            if wb_config.debug_selection: print 'Z: getUpdateUiState() list need_checkout',self.ui_state_list.need_checkout
+
+        if self.ui_state_focus is None:
+            if self.event_handler is None:
+                if wb_config.debug_selection: print 'Z: getUpdateUiState() event_handler is None set tree'
+                self.ui_state_focus = self.ui_state_tree
+            elif self.event_handler.isTreeHandler():
+                if wb_config.debug_selection: print 'Z: getUpdateUiState() event_handler is Tree set tree'
+                self.ui_state_focus = self.ui_state_tree
+            else:
+                if wb_config.debug_selection: print 'Z: getUpdateUiState() event_handler is List set list'
+                self.ui_state_focus = self.ui_state_list
+            if wb_config.debug_selection: print 'Z: getUpdateUiState() focus need_checkout',self.ui_state_focus.need_checkout
 
     def clearUpdateUiState( self ):
+        if wb_config.debug_selection: print 'Z: clearUpdateUiState()'
         self.ui_state_tree = None
         self.ui_state_list = None
         self.ui_state_focus = None
 
-    def returnNone( self ):
-        return None
 
 #--------------------------------------------------------------------------------
 class LogCtrlPanel(wx.Panel):
@@ -759,15 +1030,13 @@ class LogCtrlPanel(wx.Panel):
         self.text_ctrl = StyledLogCtrl( self.app, self )
 
         # Redirect the console IO to this panel
-        if self.app.isStdIoRedirect():
-            sys.stdin = file( wb_platform_specific.getNullDevice(), 'r' )
-            sys.stdout = self
-            sys.stderr = self
+        sys.stdin = file( wb_platform_specific.getNullDevice(), 'r' )
+        sys.stdout = self
+        sys.stderr = self
 
         # Redirect log to the Log panel
-        if self.app.isStdIoRedirect():
-            log_handler = LogHandler( self.text_ctrl )
-            self.app.log.addHandler( log_handler )
+        log_handler = LogHandler( self.text_ctrl )
+        self.app.log.addHandler( log_handler )
 
         wx.EVT_SIZE( self, wb_exceptions.TryWrapper( self.app.log, self.OnSize ) )
 
@@ -795,6 +1064,9 @@ class LogCtrlPanel(wx.Panel):
         else:
             self.text_ctrl.WriteNormal(string)
 
+        if not self.app.isStdIoRedirect():
+            sys.__stdout__.write( string )
+
     def close( self ):
         pass
 
@@ -851,10 +1123,10 @@ class StyledLogCtrl(wx.stc.StyledTextCtrl):
         self.StyleSetSpec( wx.stc.STC_STYLE_DEFAULT, 
                 "size:%d,face:%s,fore:#000000" % (point_size,face) )
 
-        self.StyleSetSpec( self.style_normal,    "fore:#000000" )
+        self.StyleSetSpec( self.style_normal,   "fore:#000000" )
         self.StyleSetSpec( self.style_error,    "fore:#DC143C" )    # Crimson
-        self.StyleSetSpec( self.style_info,    "fore:#191970" )    # Midnight Blue
-        self.StyleSetSpec( self.style_warning,    "fore:#008000" )    # Green
+        self.StyleSetSpec( self.style_info,     "fore:#191970" )    # Midnight Blue
+        self.StyleSetSpec( self.style_warning,  "fore:#008000" )    # Green
         self.StyleSetSpec( self.style_critical, "fore:#BA55D3" )    # Medium Orchid
         self.StyleSetSpec( self.style_debug,    "fore:#DC143C" )    # Crimson
 
