@@ -1,7 +1,7 @@
 '''
 
  ====================================================================
- Copyright (c) 2003-2005 Barry A Scott.  All rights reserved.
+ Copyright (c) 2003-2006 Barry A Scott.  All rights reserved.
 
  This software is licensed as described in the file LICENSE.txt,
  which you should have received as part of this distribution.
@@ -202,14 +202,19 @@ class BookmarksPreferences(PreferenceSection):
         self.app = app
 
         self.all_bookmarks = {}
-        self.menu_style = 'leaf_only'
-        self.leaf_names_to_ignore = ['main','source','src', 'inc','include']
+        # no longer support menu style -
+        # self.menu_style = 'leaf_only'
+        # self.leaf_names_to_ignore = ['main','source','src', 'inc','include']
+        self.menu_style = None
+        self.leaf_names_to_ignore = []
 
     def readPreferences( self, pref_data ):
         if not pref_data.has_section( self.section_name ):
             return
 
         get_option = GetOption( pref_data, self.section_name )
+        # look for menu_style and leaf_names_to_ignore to
+        # allow for preference file update
         if get_option.has( 'menu_style' ):
             self.menu_style = get_option.getstr( 'menu_style' )
 
@@ -226,6 +231,15 @@ class BookmarksPreferences(PreferenceSection):
                 provider = wb_source_control_providers.getProvider( provider )
                 pi = provider.getProjectInfo( self.app )
                 pi.readPreferences( get_option )
+                # default the menu name if required
+                if pi.menu_name is None:
+                    # see if update required
+                    if self.menu_style is not None:
+                        # default the name as if used to be done
+                        pi.menu_name = self.__getMenuName( pi.wc_path )
+                    else:
+                        self.__defaultMenuName( pi )
+
                 self.all_bookmarks[ bookmark_name ] = pi
 
     def writePreferences( self, pref_data ):
@@ -233,8 +247,8 @@ class BookmarksPreferences(PreferenceSection):
         pref_data.add_section( self.section_name )
 
         set_option = SetOption( pref_data, self.section_name )
-        set_option.set( 'menu_style', self.menu_style )
-        set_option.set( 'leaf_names_to_ignore', self.leaf_names_to_ignore, ',' )
+        #set_option.set( 'menu_style', self.menu_style )
+        #set_option.set( 'leaf_names_to_ignore', self.leaf_names_to_ignore, ',' )
 
         set_option.set( 'num_bookmarks', len( self.all_bookmarks ) )
         for index, bookmark_name in enumerate( self.all_bookmarks.keys() ):
@@ -243,8 +257,11 @@ class BookmarksPreferences(PreferenceSection):
             pi = self.all_bookmarks[ bookmark_name ]
             pi.writePreferences( set_option )
 
-    def addBookmark( self, bookmark_name, pi ):
-        self.all_bookmarks[ bookmark_name ] = pi
+    def addBookmark( self, pi, name=None ):
+        if name is None:
+            name = pi.wc_path
+        self.all_bookmarks[ name ] = pi
+        self.__defaultMenuName( pi )
 
     def delBookmark( self, bookmark_name ):
         del self.all_bookmarks[ bookmark_name ]
@@ -257,12 +274,28 @@ class BookmarksPreferences(PreferenceSection):
         names.sort()
         return names
 
-    def getMenuName( self, wc_path ):
-        if self.menu_style in ['leaf_in_parent', 'leaf_only']:
+    def __defaultMenuName( self, pi ):
+        if os.environ.has_key( 'HOME' ):
+            home_dir = os.environ[ 'HOME' ] + '/'
+            if pi.wc_path.startswith( home_dir ):
+                pi.menu_name = pi.wc_path[len(home_dir):]
+            else:
+                pi.menu_name = pi.wc_path
+        else:
+            pi.menu_name = pi.wc_path
+
+    # only used to update pref file now
+    def __getMenuName( self, wc_path, menu_style=None, leaf_names_to_ignore=None ):
+        if menu_style is None:
+            menu_style = self.menu_style
+        if leaf_names_to_ignore is None:
+            leaf_names_to_ignore = self.leaf_names_to_ignore
+
+        if menu_style in ['leaf_in_parent', 'leaf_only']:
             path_parts = wc_path.split( os.path.sep )
             for leaf_index in range( len(path_parts)-1, 1, -1 ):
-                if path_parts[ leaf_index ].lower() not in self.leaf_names_to_ignore:
-                    if self.menu_style == 'leaf_in_parent':
+                if path_parts[ leaf_index ].lower() not in leaf_names_to_ignore:
+                    if menu_style == 'leaf_in_parent':
                         return '%s   in %s' % (os.path.sep.join( path_parts[leaf_index:] )
                                               ,os.path.sep.join( path_parts[:leaf_index] ))
                     else:
