@@ -1,6 +1,6 @@
 '''
  ====================================================================
- Copyright ( c ) 2003-2005 Barry A Scott.  All rights reserved.
+ Copyright ( c ) 2003-2006 Barry A Scott.  All rights reserved.
 
  This software is licensed as described in the file LICENSE.txt,
  which you should have received as part of this distribution.
@@ -13,7 +13,7 @@
 import wx
 import wb_exceptions
 import os
-import wb_subversion_list_handler
+import wb_subversion_list_handler_common
 import wb_shell_commands
 
 class PreferencesDialog( wx.Dialog ):
@@ -96,7 +96,6 @@ class PagePanel(wx.Panel):
     def initControls( self ):
         raise wb_exceptions.InternalError('must override initControls')
 
-browse_button_id = wx.NewId()
 class EditorPage(PagePanel):
     def __init__( self, notebook, app ):
         self.app = app
@@ -111,20 +110,20 @@ class EditorPage(PagePanel):
         self.static_text2 = wx.StaticText( self, -1, "Edit Arguments: ", style=wx.ALIGN_RIGHT)
         self.text_ctrl_edit_arg = wx.TextCtrl( self, -1, p.editor_options, wx.DefaultPosition, wx.Size(315, -1) )
 
-        self.browse_button = wx.Button( self, browse_button_id, " Browse... ")
+        self.browse_button = wx.Button( self, -1, " Browse... ")
 
         self.grid_sizer = wx.FlexGridSizer( 0, 3, 0, 0 )
         self.grid_sizer.AddGrowableCol( 1 )
 
-        self.grid_sizer.Add( self.static_text1, 1, wx.EXPAND|wx.NORTH, 3 )
-        self.grid_sizer.Add( self.text_ctrl_editor, 0, wx.EXPAND|wx.EAST, 5 )
+        self.grid_sizer.Add( self.static_text1, 1, wx.EXPAND|wx.ALL, 3 )
+        self.grid_sizer.Add( self.text_ctrl_editor, 0, wx.EXPAND|wx.ALL, 5 )
         self.grid_sizer.Add( self.browse_button, 0, wx.EXPAND )
 
-        self.grid_sizer.Add( self.static_text2, 1, wx.EXPAND|wx.NORTH, 3 )
-        self.grid_sizer.Add( self.text_ctrl_edit_arg, 0, wx.EXPAND|wx.EAST, 5 )
+        self.grid_sizer.Add( self.static_text2, 1, wx.EXPAND|wx.ALL, 3 )
+        self.grid_sizer.Add( self.text_ctrl_edit_arg, 0, wx.EXPAND|wx.ALL, 5 )
         self.grid_sizer.Add( (1, 1), 0, wx.EXPAND )
 
-        wx.EVT_BUTTON( self, browse_button_id, self.OnBrowseExe )
+        wx.EVT_BUTTON( self, self.browse_button.GetId(), self.OnBrowseExe )
 
         return self.grid_sizer
 
@@ -136,7 +135,7 @@ class EditorPage(PagePanel):
 
 
     def OnBrowseExe( self, event ):
-        if wx.Platform == '__WXWIN__':
+        if wx.Platform == '__WXMSW__':
             file_type = "Executable files (*.exe)|*.exe"
         elif wx.Platform == '__WXMAC__':
             file_type = "Applications|*.app|Executable files|*"
@@ -167,7 +166,7 @@ class EditorPage(PagePanel):
         if wx.Platform == '__WXMAC__':
             valid = (os.access( self.text_ctrl_editor.GetValue(), os.X_OK )
                 or not os.path.isdir( self.text_ctrl_editor.GetValue() ) )
-        elif wx.Platform == '__WXWIN__':
+        elif wx.Platform == '__WXMSW__':
             valid = (os.path.exists( self.text_ctrl_editor.GetValue() )
                 or not os.path.isdir( self.text_ctrl_editor.GetValue() ) )
         else:
@@ -206,13 +205,13 @@ class ShellPage(PagePanel):
                 self.terminal_program_list_ctrl.SetStringSelection( p.shell_terminal )
             else:
                 self.terminal_program_list_ctrl.SetStringSelection( terminal_program_list[0] )
-            self.grid_sizer.Add( self.static_text2, 1, wx.EXPAND|wx.NORTH, 5 )
-            self.grid_sizer.Add( self.terminal_program_list_ctrl, 0, wx.EXPAND|wx.EAST, 5 )
+            self.grid_sizer.Add( self.static_text2, 1, wx.EXPAND|wx.ALL, 3 )
+            self.grid_sizer.Add( self.terminal_program_list_ctrl, 0, wx.EXPAND|wx.ALL, 3 )
         else:
             self.terminal_program_list_ctrl = None
 
-        self.grid_sizer.Add( self.static_text1, 1, wx.EXPAND|wx.NORTH, 5 )
-        self.grid_sizer.Add( self.text_ctrl_shell_init_command, 0, wx.EXPAND|wx.EAST, 5 )
+        self.grid_sizer.Add( self.static_text1, 1, wx.EXPAND|wx.ALL, 3 )
+        self.grid_sizer.Add( self.text_ctrl_shell_init_command, 0, wx.EXPAND|wx.ALL, 3 )
 
         file_browser_program_list = wb_shell_commands.getFileBrowserProgramList()
 
@@ -223,8 +222,8 @@ class ShellPage(PagePanel):
                 self.file_browser_program_list_ctrl.SetStringSelection( p.shell_file_browser )
             else:
                 self.file_browser_program_list_ctrl.SetStringSelection( file_browser_program_list[0] )
-            self.grid_sizer.Add( self.static_text3, 1, wx.EXPAND|wx.NORTH, 5 )
-            self.grid_sizer.Add( self.file_browser_program_list_ctrl, 0, wx.EXPAND|wx.EAST, 5 )
+            self.grid_sizer.Add( self.static_text3, 1, wx.EXPAND|wx.ALL, 3 )
+            self.grid_sizer.Add( self.file_browser_program_list_ctrl, 0, wx.EXPAND|wx.ALL, 3 )
         else:
             self.file_browser_program_list_ctrl = None
 
@@ -377,7 +376,7 @@ class ListColumnsPage(PagePanel):
     def __init__( self, notebook, app ):
         self.app = app
         self.selected_name = None
-        self.column_info = wb_subversion_list_handler.ViewColumnInfo()
+        self.column_info = wb_subversion_list_handler_common.ViewColumnInfo()
         PagePanel.__init__( self, notebook, 'Columns' )
 
     def initControls( self ):
