@@ -1,6 +1,6 @@
 '''
  ====================================================================
- Copyright (c) 2005 Barry A Scott.  All rights reserved.
+ Copyright (c) 2005-2006 Barry A Scott.  All rights reserved.
 
  This software is licensed as described in the file LICENSE.txt,
  which you should have received as part of this distribution.
@@ -20,6 +20,19 @@ def readFileContentsAsUnicode( filename ):
     return contentsAsUnicode( contents )
 
 def contentsAsUnicode( contents ):
+    encoding = encodingFromContents( contents )
+
+    try:
+        return contents.decode( encoding )
+    except UnicodeDecodeError:
+        # fall back to latin-1
+        try:
+            return contents.decode( 'iso8859-1' )
+        except UnicodeDecodeError:
+            # sigh this is hard. use the choosen encoding and replace chars in error
+            return contents.decode( encoding, 'replace' )
+
+def encodingFromContents( contents ):
     if( len(contents) > len(codecs.BOM_UTF8)
     and contents[0:len(codecs.BOM_UTF8)] == codecs.BOM_UTF8 ):
         encoding = 'utf-8'
@@ -33,6 +46,6 @@ def contentsAsUnicode( contents ):
         encoding = locale.getdefaultlocale()[1]
 
     if encoding is None:
-        return contents
+        encoding = 'iso8859-1'
 
-    return contents.decode( encoding )
+    return encoding
