@@ -1,6 +1,6 @@
 '''
  ====================================================================
- Copyright (c) 2003-2005 Barry A Scott.  All rights reserved.
+ Copyright (c) 2003-2006 Barry A Scott.  All rights reserved.
 
  This software is licensed as described in the file LICENSE.txt,
  which you should have received as part of this distribution.
@@ -14,7 +14,7 @@ import os
 import xml.sax.saxutils
 
 def getTerminalProgramList():
-    return ['Terminal']
+    return ['Terminal','iTerm']
 
 def getFileBrowserProgramList():
     return ['Finder']
@@ -50,6 +50,67 @@ def ShellOpen( app, project_info, filename ):
         os.chdir( cur_dir )
 
 def CommandShell( app, project_info ):
+    p = app.prefs.getShell()
+    if p.shell_terminal == 'iTerm':
+        CommandShell_iTerm( app, project_info )
+    else:
+        CommandShell_Terminal( app, project_info )
+
+def CommandShell_iTerm( app, project_info ):
+    shell_script_filename = os.path.join( '/tmp', 'wb.scpt' )
+
+    p = app.prefs.getShell()
+    working_dir = project_info.getWorkingDir()
+
+    # calc a title that is leaf to root so that the leaf shows up in a task bar first
+    title = []
+    pi = project_info
+    while pi:
+        title.append( pi.project_name )
+        pi = pi.parent
+
+    commands = 'cd "%s"' % working_dir
+
+    if len( p.shell_init_command ) > 0:
+        commands = commands + ';. "%s"\n' % p.shell_init_command
+
+    f  = open( shell_script_filename, 'w' )
+    f.write( '''
+tell application "iTerm"
+    activate 
+
+    -- make a new terminal
+    set work_bench_term to (make new terminal) 
+
+    -- talk to the new terminal
+    tell work_bench_term 
+        activate current session
+        launch session "Default Session"
+
+        -- talk to the session
+        tell the last session
+            set name to "%s"
+
+            -- execute a command
+            exec command "/bin/bash"
+
+            write text "%s"
+
+        end tell
+
+    end tell
+
+end
+''' %   (' '.join( title ).replace( '"', '\\"' )
+        ,commands.replace( '"', '\\"' )) )
+    f.close()
+
+    command_line = '"osascript" "%s" &' % shell_script_filename
+
+    app.log.info( command_line )
+    os.system( command_line )
+
+def CommandShell_Terminal( app, project_info ):
     shell_script_filename = os.path.join( '/tmp', 'wb.term' )
 
     p = app.prefs.getShell()
