@@ -1,13 +1,13 @@
 '''
  ====================================================================
- Copyright (c) 2003-2005 Barry A Scott.  All rights reserved.
+ Copyright (c) 2003-2006 Barry A Scott.  All rights reserved.
 
  This software is licensed as described in the file LICENSE.txt,
  which you should have received as part of this distribution.
 
  ====================================================================
 
-    wb_subversion_provider.py
+    wb_subversion_project_info.py
 
 '''
 import sys
@@ -26,7 +26,7 @@ import wb_subversion_tree_handler
 import wb_subversion_list_handler
 import wb_subversion_utils
 
-_debug_performance = False
+_fast_proplist = True
 
 class ErrorWrapper:
     def __init__( self, args ):
@@ -50,7 +50,8 @@ class ProjectInfo(wb_source_control_providers.ProjectInfo):
         self.tags_url = None
 
     def __repr__( self ):
-        return '<wb_subversion_provider.ProjectInfo url=%s, wc_path=%s>' % (self.url, self.wc_path)
+        return '<svn.ProjectInfo wc_path=%s>' % self.wc_path
+        #return '<wb_subversion_provider.ProjectInfo url=%s, wc_path=%s>' % (self.url, self.wc_path)
 
     def init( self, project_name, **kws):
         wb_source_control_providers.ProjectInfo.init( self, project_name )
@@ -144,7 +145,9 @@ class ProjectInfo(wb_source_control_providers.ProjectInfo):
         self.app.foregroundProcess( sys.stdout.write, (msg,) )
 
     def readPreferences( self, get_option ):
-        # load state into a preference file
+        wb_source_control_providers.ProjectInfo.readPreferences( self, get_option )
+        
+        # load state from a preference file
         name = get_option.getstr( 'name' )
         url = get_option.getstr( 'url' )
         wc_path = get_option.getstr( 'wc_path' )
@@ -201,6 +204,8 @@ class ProjectInfo(wb_source_control_providers.ProjectInfo):
         self.need_properties = state
 
     def updateStatus( self ):
+        self.app.log.debug( 'updateStatus() %r' % self )
+
         self.all_files_status = []
         self.files_properties = {}
         self.dir_status = None
@@ -210,7 +215,6 @@ class ProjectInfo(wb_source_control_providers.ProjectInfo):
             return
 
         p = self.app.prefs.getView()
-        if _debug_performance: print '================================================================================'
         try:
             entry = self.client_fg.info( self.wc_path )
         except pysvn.ClientError, e:
@@ -224,10 +228,7 @@ class ProjectInfo(wb_source_control_providers.ProjectInfo):
         else:
             self.url = entry.url
 
-        t0 = time.time()
         self.all_files_status = self.client_fg.status( self.wc_path, recurse=p.view_recursive, ignore=False )
-        t1 = time.time()
-        if _debug_performance: print 'updateStatus: %6.3f - %-8s %s' % (t1-t0, 'status', self.wc_path)
 
         self.need_checkout = False
 
@@ -242,18 +243,20 @@ class ProjectInfo(wb_source_control_providers.ProjectInfo):
         self.all_files_status = self.filterFilenames( self.all_files_status )
 
         if self.need_properties:
-            try:
-                name_list = [file.path for file in self.all_files_status if file.is_versioned]
-                t0 = time.time()
-                path_properties = self.client_fg.proplist( name_list, recurse=False )
-                t1 = time.time()
-                if _debug_performance: print 'updateStatus: %6.3f - %-8s %s' % (t1-t0, 'proplist', self.wc_path)
+            if _fast_proplist:
+                for status in self.all_files_status:
+                    if status.is_versioned:
+                        self.files_properties[ status.path ] = self.__proplist( status.path )
+            else:
+                try:
+                    name_list = [status.path for status in self.all_files_status if status.is_versioned]
+                    path_properties = self.client_fg.proplist( name_list, recurse=False )
 
-                for path, prop_dict in path_properties:
-                    self.files_properties[ path ] = prop_dict
+                    for path, prop_dict in path_properties:
+                        self.files_properties[ path ] = prop_dict
 
-            except pysvn.ClientError, e:
-                pass
+                except pysvn.ClientError, e:
+                    pass
 
         if p.view_recursive:
             wc_path_num_parts = len( self.wc_path.split( os.sep ) ) + 1
@@ -261,7 +264,7 @@ class ProjectInfo(wb_source_control_providers.ProjectInfo):
             for file in self.all_files_status:
                 if len( file.path.split( os.sep ) ) == wc_path_num_parts:
                     self.all_tree_files_status.append( file )
-                
+
         else:
             self.all_tree_files_status = self.all_files_status
 
@@ -300,6 +303,36 @@ class ProjectInfo(wb_source_control_providers.ProjectInfo):
     def getDirStatus( self ):
         return self.dir_status
 
+    def __proplist( self, path ):
+        if os.path.isdir( path ):
+            prop_file = os.path.join( path, '.svn', 'dir-props' )
+        else:
+            dirname, basename = os.path.split( path )
+            prop_file = os.path.join( dirname, '.svn', 'props', basename + '.svn-work' )
+
+        result = {}
+        try:
+            f = file( prop_file )
+        except EnvironmentError:
+            return result
+
+        while True:
+            line = f.readline()
+            if line == '':
+                break
+            if line == 'END\n':
+                break
+            code, length = line.split()
+            body = f.read( int(length)+1 )
+            if code == 'K':
+                key = body[:-1]
+            elif code == 'V':
+                result[ key ] = body[:-1]
+            else:
+                raise ValueError( 'Unparsed line %s' % line )
+
+        f.close()
+        return result
 
 #
 #    Used to allow a call to function on the background thread
