@@ -1,6 +1,6 @@
 '''
  ====================================================================
- Copyright (c) 2003-2005 Barry A Scott.  All rights reserved.
+ Copyright (c) 2003-2006 Barry A Scott.  All rights reserved.
 
  This software is licensed as described in the file LICENSE.txt,
  which you should have received as part of this distribution.
@@ -20,7 +20,13 @@ import wb_subversion_project_info
 import wb_subversion_utils
 import wb_subversion_info_dialog
 import wb_subversion_history
+import wb_subversion_checkin
+import wb_subversion_report_updates
+import wb_subversion_report_lock
+import wb_subversion_report_branch_changes
 import wb_subversion_properties_dialog
+import wb_clipboard
+import wb_dialogs
 
 class SubversionProject(wb_tree_panel.TreeProjectItem):
     def __init__( self, app, project_info ):
@@ -112,6 +118,7 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
             state.versioned = True
             state.unversioned = False
             state.need_checkin = False
+            state.need_checkout = True
             state.conflict = False
             state.file_exists = False
         else:
@@ -120,6 +127,7 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
             state.versioned = True
             state.unversioned = True
             state.need_checkin = True
+            state.need_checkout = False
             state.conflict = True
             state.file_exists = True
 
@@ -156,7 +164,7 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
         menu_item =[('', wb_ids.id_Command_Shell, '&Command Shell')
             ,('', wb_ids.id_File_Browser, '&File Browser')
             ,('-', 0, 0 )
-            ,('', wb_ids.id_SP_History, 'History...' )
+            ,('', wb_ids.id_SP_History, 'Log history...' )
             ,('', wb_ids.id_SP_Info, 'Information...' )
             ,('', wb_ids.id_SP_Properties, 'Properties...' )
             ,('-', 0, 0 )]
@@ -167,18 +175,33 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
         menu_item += [('-', 0, '' )
             ,('', wb_ids.id_SP_Checkin, 'Checkin...' )
             ,('-', 0, 0 )
+            ,('', wb_ids.id_SP_NewFile, 'New File...' )
             ,('', wb_ids.id_SP_Mkdir, 'Make directory...' )
             ,('', wb_ids.id_SP_Add, 'Add...' )
             ,('', wb_ids.id_SP_Rename, 'Rename...' )
             ,('-', 0, 0 )
-            ,('', wb_ids.id_SP_Delete, 'Delete' )
-            ,('', wb_ids.id_SP_Revert, 'Revert' )
+            ,('', wb_ids.id_SP_Delete, 'Delete...' )
+            ,('', wb_ids.id_SP_Revert, 'Revert...' )
             ,('-', 0, 0 )
             ,('', wb_ids.id_SP_Cleanup, 'Clean up' )
             ]
 
         return wb_subversion_utils.populateMenu( wx.Menu(), menu_item )
 
+    def Cmd_Dir_EditCopy( self ):
+        self.app.setPasteData( wb_clipboard.Clipboard( [self.project_info.wc_path], is_copy=True ) )
+        print 'Copied folder %s to the Clipboard' % self.project_info.wc_path
+        self.app.refreshFrame()
+ 
+    def Cmd_Dir_EditCut( self ):
+        self.app.setPasteData( wb_clipboard.Clipboard( [self.project_info.wc_path], is_copy=False ) )
+        print 'Cut folder %s to the Clipboard' % self.project_info.wc_path
+        self.app.refreshFrame()
+
+    def Cmd_Dir_EditPaste( self ):
+        print 'Cmd_Dir_EditPaste'
+        self.app.frame.list_panel.OnSpEditPaste()
+
     def Cmd_Dir_Add( self ):
         name = os.path.basename( self.project_info.wc_path )
         force = self.app.addFile( 'Add Directory', name, False )
@@ -227,43 +250,37 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
     def Cmd_Dir_Checkin( self ):
         # get a status output to show the user
 
-        all_files = self.project_info.client_fg.status( self.project_info.wc_path, recurse=True, get_all=False, ignore=True, update=False )
-        status_output = self._cmd_status_output( all_files )
-        if len(status_output) == 0:
-            wx.MessageBox( "There are no changes to checkin",
+        self.app.setAction( 'Look for changes to check in %s...' % self.project_info.wc_path )
+
+        error = False
+        yield self.app.backgroundProcess
+        try:
+            all_files = self.project_info.client_bg.status(
+                self.project_info.wc_path,
+                recurse=True, get_all=False,
+                ignore=True, update=False )
+            all_files = [entry for entry in all_files
+                            if wb_subversion_utils.wc_status_checkin_map[ entry.text_status ]
+                            or wb_subversion_utils.wc_status_checkin_map[ entry.prop_status ]]
+        except pysvn.ClientError, e:
+            self.app.log_client_error( e )
+            error = True
+        yield self.app.foregroundProcess
+
+        self.app.setAction( 'Ready' )
+
+        if error:
+            return
+
+        if len(all_files) == 0:
+            wx.MessageBox( "There are no changes to check in",
                 "Warning", style=wx.OK|wx.ICON_EXCLAMATION )
             return
 
-        message = self.app.getLogMessage( 'Check in', status_output )
+        ci_frame = wb_subversion_checkin.CheckinFrame( self.app, self.project_info, all_files )
+        ci_frame.Show( True )
 
-        if not message:
-            return
-
-        self.app.setAction( 'Check in %s...' % self.project_info.wc_path )
-        self.app.setProgress( 'Sent %(count)d of %(total)d', len( status_output ) )
-
-        yield self.app.backgroundProcess
-
-        ok = False
-        try:
-            rev = self.project_info.client_bg.checkin( self.project_info.wc_path, message )
-            ok = True
-        except pysvn.ClientError, e:
-            self.app.log_client_error( e )
-
-        yield self.app.foregroundProcess
-
-        if ok:
-            if rev:
-                self.app.log.info( 'Checkin created revision %d' % rev.number )
-            else:
-                self.app.log.warning( 'No changes to checkin ' )
-
-        self.app.refreshFrame()
-        self.app.setAction( 'Ready' )
-        self.app.clearProgress()
-
-    def _cmd_status_output( self, all_files ):
+    def __cmd_status_output( self, all_files ):
         output_lines = []
         all_files.sort( wb_subversion_utils.by_path )
         for file in all_files:
@@ -285,35 +302,48 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
 
     def Cmd_Dir_History( self ):
         filename = self.project_info.wc_path
-        self.app.setAction( 'History %s...' % filename )
+
+        dialog = wb_subversion_history.HistoryDialog( self.app.frame.tree_panel.tree_ctrl )
+        result = dialog.ShowModal()
+        if result != wx.ID_OK:
+            return
+
+        self.app.setAction( 'Log history %s...' % filename )
 
         yield self.app.backgroundProcess
 
         ok = False
         history_entries = []
         try:
-                log_entries = self.project_info.client_bg.log( filename, strict_node_history=True )
-                for log in log_entries:
-                    history_entries.append( (log['revision'].number,
-                        log['author'],
-                        log['date'],
+            log_entries = self.project_info.client_bg.log(
+                            filename,
+                            strict_node_history=False,
+                            discover_changed_paths=True,
+                            limit=dialog.getLimit(),
+                            revision_end=dialog.getRevisionEnd() )
+            for log in log_entries:
+                history_entries.append( (log['revision'].number,
+                    log['author'],
+                    log['date'],
+                    '',
+                    log['message'],
+                    log['changed_paths']) )
+            info = self.project_info.client_bg.info( filename )
+
+            tags_url = self.project_info.getTagsUrl( info.url )
+            if tags_url:
+                for ls_info in self.project_info.client_bg.ls( tags_url ):
+                    history_entries.append( (ls_info['created_rev'].number,
+                        ls_info['last_author'],
+                        ls_info['time'],
+                        'Tag ' + ls_info['name'].split('/')[-1],
                         '',
-                        log['message'] ) )
-                info = self.project_info.client_bg.info( filename )
-
-                tags_url = self.project_info.getTagsUrl( info.url )
-                if tags_url:
-                    for ls_info in self.project_info.client_bg.ls( tags_url ):
-                        history_entries.append( (ls_info['created_rev'].number,
-                            ls_info['last_author'],
-                            ls_info['time'],
-                            'Tag ' + ls_info['name'].split('/')[-1],
-                            '' ) )
+                        [] ) )
 
 
-                history_entries.sort()
+            history_entries.sort()
 
-                ok = True
+            ok = True
         except pysvn.ClientError, e:
             self.app.log_client_error( e )
 
@@ -324,6 +354,7 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
             h_frame.Show( True )
 
         self.app.setAction( 'Ready' )
+        self.app.refreshFrame()
 
     def Cmd_Dir_Info( self ):
         try:
@@ -355,6 +386,42 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
         # open up the current tree node to show the created item
         self.app.expandSelectedTreeNode()
 
+    def Cmd_Dir_NewFile( self ):
+        pi = self.app.frame.tree_panel.getProjectTopProjectInfo()
+
+        template_dir = ''
+        if pi is not None:
+            template_dir = pi.new_file_template_dir
+
+        dialog = wb_dialogs.NewFile( self.app.frame.tree_panel.tree_ctrl, template_dir )
+        if dialog.ShowModal() == wx.ID_OK:
+            template_contents = ''
+            if template_dir != '':
+                try:
+                    template_filename = os.path.join( template_dir, dialog.getTemplateFilename() )
+                    t = file( template_filename, 'r' )
+                    template_contents = t.read()
+                    t.close()
+                except EnvironmentError, e:
+                    self.log.error( 'Cannot read template %s - %s' % (template_filename, str(e)) )
+                    return
+
+            try:
+                new_filename = os.path.join( self.project_info.wc_path, dialog.getNewFilename() )
+                f = file( new_filename, 'w' )
+                f.write( template_contents )
+                f.close()
+            except EnvironmentError, e:
+                self.log.error( 'Cannot create new file %s - %s' % (new_filename, str(e)) )
+                return
+
+            try:
+                self.project_info.client_fg.add( new_filename )
+            except pysvn.ClientError, e:
+                self.app.log_client_error( e )
+
+        self.app.refreshFrame()
+
     def Cmd_Dir_Properties( self ):
         client_fg = self.project_info.client_fg
 
@@ -385,6 +452,131 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
 
         self.app.refreshFrame()
 
+    def Cmd_Dir_ReportLocksWc( self ):
+        return self.__reportLocksCommon( show_repos_locks=False )
+
+    def Cmd_Dir_ReportLocksRepos( self ):
+        return self.__reportLocksCommon( show_repos_locks=True )
+
+    def __reportLocksCommon( self, show_repos_locks ):
+
+        if show_repos_locks:
+            self.app.setAction( 'Looking for repository locks in %s...' % self.project_info.wc_path )
+        else:
+            self.app.setAction( 'Looking for working copy locks in %s...' % self.project_info.wc_path )
+
+        ok = True
+        yield self.app.backgroundProcess
+        try:
+            all_files = self.project_info.client_bg.status(
+                self.project_info.wc_path,
+                recurse=True, get_all=False,
+                ignore=True, update=show_repos_locks )
+            all_files = [status for status in all_files
+                            if (status.entry is not None and status.entry.lock_token is not None)
+                            or (status.repos_lock is not None)]
+        except pysvn.ClientError, e:
+            self.app.log_client_error( e )
+            ok = False
+        yield self.app.foregroundProcess
+
+        self.app.setAction( 'Ready' )
+
+        if not ok:
+            return
+
+        if len(all_files) == 0:
+            if show_repos_locks:
+                wx.MessageBox( "There are no locked files in the repository",
+                    "Warning", style=wx.OK|wx.ICON_EXCLAMATION )
+            else:
+                wx.MessageBox( "There are no locked files in the working copy",
+                    "Warning", style=wx.OK|wx.ICON_EXCLAMATION )
+            return
+
+        ci_frame = wb_subversion_report_lock.ReportLockFrame(
+                        self.app, self.project_info, all_files,
+                        show_repos_locks=show_repos_locks )
+        ci_frame.Show( True )
+
+    def Cmd_Dir_ReportUpdates( self ):
+        self.app.setAction( 'Updates %s...' % self.project_info.url )
+
+        ok = False
+        all_files = []
+
+        yield self.app.backgroundProcess
+        try:
+            all_files = self.project_info.client_bg.status(
+                self.project_info.wc_path,
+                recurse=True, get_all=False,
+                ignore=True, update=True )
+            ok = True
+
+        except pysvn.ClientError, e:
+            self.app.log_client_error( e )
+
+        yield self.app.foregroundProcess
+
+        if len(all_files) == 0:
+            wx.MessageBox( "All files are up todate",
+                "Warning", style=wx.OK|wx.ICON_EXCLAMATION )
+            return
+
+        self.app.setAction( 'Ready' )
+
+        if ok:
+            all_files = [status for status in all_files
+                            if status.repos_prop_status != pysvn.wc_status_kind.none
+                            or (status.entry is not None
+                                and status.entry.kind == pysvn.node_kind.file
+                                and status.repos_text_status != pysvn.wc_status_kind.none)]
+            ci_frame = wb_subversion_report_updates.ReportUpdatesFrame( self.app, self.project_info, all_files )
+            ci_frame.Show( True )
+
+    def Cmd_Dir_ReportBranchChanges(self):
+        self.app.setAction('Branch changes %s...' % self.project_info.url)
+
+        changed_files = []
+        is_branch = False
+
+        yield self.app.backgroundProcess
+        
+        try:
+            log_entries = self.project_info.client_bg.log(self.project_info.url, discover_changed_paths=True)
+            if log_entries[-1]['changed_paths'][0]['copyfrom_path'] is not None:
+               is_branch = True
+               all_files = self.project_info.client_bg.status(self.project_info.wc_path,
+                   recurse=True, get_all=True, ignore=True, update=False)
+               repos_root_url = self.project_info.client_bg.info2(self.project_info.wc_path)[0][1]['repos_root_URL']
+               changed_urls = []
+               for log_entry in log_entries:
+                   for changed_paths in log_entry['changed_paths']:
+                       url = repos_root_url + changed_paths['path']
+                       if not (url in changed_urls):
+                          changed_urls.append(url)
+               for status in all_files:
+                   if (status.entry is not None 
+                       and status.entry.kind == pysvn.node_kind.file
+                       and (status.text_status != pysvn.wc_status_kind.normal or 
+                            status.entry.url in changed_urls)):
+                       changed_files.append(status)
+        except pysvn.ClientError, e:
+            self.app.log_client_error(e)
+
+        yield self.app.foregroundProcess
+
+        if not is_branch:
+            wx.MessageBox('"%s" is not a branch.' % self.project_info.url, "Error", style=wx.OK|wx.ICON_ERROR)
+        elif len(changed_files) == 0:
+            wx.MessageBox('No files changed yet in branch "%s".', "Error", style=wx.OK|wx.ICON_ERROR)
+        else:
+            ci_frame = wb_subversion_report_branch_changes.ReportBranchChangesFrame(self.app, self.project_info, changed_files)
+            ci_frame.Show(True)
+
+        self.app.setAction('Ready')
+
+
     def Cmd_Dir_Rename( self ):
         old_filename = self.project_info.wc_path
         old_name = os.path.basename( old_filename )
@@ -415,7 +607,7 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
     def Cmd_Dir_Revert( self ):
         # get a status output to show the user
         all_files = self.project_info.client_fg.status( self.project_info.wc_path, recurse=True, get_all=False, ignore=True, update=False )
-        status_output = self._cmd_status_output( all_files )
+        status_output = self.__cmd_status_output( all_files )
         if len(status_output) == 0:
             wx.MessageBox( "There are no changes to revert",
                 "Warning", style=wx.OK|wx.ICON_EXCLAMATION )
@@ -435,9 +627,10 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
 
         yield self.app.backgroundProcess
 
+        filename = self.project_info.wc_path
         ok = False
         try:
-            rev = self.project_info.client_bg.update( self.project_info.wc_path, recurse=True )
+            rev_list = self.project_info.client_bg.update( filename, recurse=True )
             ok = True
         except pysvn.ClientError, e:
             self.app.log_client_error( e )
@@ -445,16 +638,17 @@ class SubversionProject(wb_tree_panel.TreeProjectItem):
         yield self.app.foregroundProcess
 
         if ok:
-            if rev.number > 0:
-                count = self.app.getProgressValue( 'count' )
-                if count == 0:
-                    self.app.log.info( 'Updated to revision %d, no new updates' % rev.number )
-                elif count == 1:
-                    self.app.log.info( 'Updated to revision %d, 1 new update' % rev.number )
+            for rev in rev_list:
+                if rev.number > 0:
+                    count = self.app.getProgressValue( 'count' )
+                    if count == 0:
+                        self.app.log.info( 'Updated %s to revision %d, no new updates' % (filename, rev.number) )
+                    elif count == 1:
+                        self.app.log.info( 'Updated %s to revision %d, 1 new update' % (filename, rev.number) )
+                    else:
+                        self.app.log.info( 'Updated %s to revision %d, %d new updates' % (filename, rev.number, count) )
                 else:
-                    self.app.log.info( 'Updated to revision %d, %d new updates' % (rev.number, count) )
-            else:
-                self.app.log.warning( 'Already up to date' )
+                    self.app.log.warning( 'Already up to date' )
 
         self.app.clearProgress()
         self.app.setAction( 'Ready' )
