@@ -1,6 +1,6 @@
 '''
  ====================================================================
- Copyright (c) 2003-2005 Barry A Scott.  All rights reserved.
+ Copyright (c) 2003-2006 Barry A Scott.  All rights reserved.
 
  This software is licensed as described in the file LICENSE.txt,
  which you should have received as part of this distribution.
@@ -11,14 +11,17 @@
 
 '''
 import wx
+import cPickle
+import os
+
 import wb_exceptions
 import wb_source_control_providers
 import wb_ids
 import wb_shell_commands
 import wb_dialogs
 import wb_project_dialogs
-import cPickle
-import os
+
+import wb_config
 
 class TreeState:
     def __init__( self ):
@@ -26,6 +29,7 @@ class TreeState:
         self.versioned = False
         self.unversioned = False
         self.need_checkin = False
+        self.need_checkout = False
         self.conflict = False
         self.file_exists = False
         self.is_project_parent = False
@@ -57,12 +61,66 @@ class WbTreePanel(wx.Panel):
 
         self.tree_ctrl = WbTreeCtrl( self, self.app )
 
+        if wx.Platform == '__WXMAC__':
+            acc_mod = wx.ACCEL_ALT
+
+            acc_init =[
+                (wx.ACCEL_ALT, ord('C'), wb_ids.id_SP_EditCopy),
+                (wx.ACCEL_ALT, ord('X'), wb_ids.id_SP_EditCut),
+                (wx.ACCEL_ALT, ord('V'), wb_ids.id_SP_EditPaste),
+
+                (wx.ACCEL_ALT, ord('A'), wb_ids.id_SP_Add),
+                (wx.ACCEL_ALT, ord('L'), wb_ids.id_SP_History),
+                (wx.ACCEL_ALT, ord('I'), wb_ids.id_SP_Info),
+                (wx.ACCEL_ALT, ord('P'), wb_ids.id_SP_Properties),
+                (wx.ACCEL_ALT, ord('R'), wb_ids.id_SP_Revert),
+                (wx.ACCEL_ALT, ord('U'), wb_ids.id_SP_Update),
+                (wx.ACCEL_NORMAL, wx.WXK_DELETE, wb_ids.id_SP_Delete),
+                (wx.ACCEL_NORMAL, wx.WXK_RETURN, wb_ids.id_Return_Hotkey),
+                ]
+        elif wx.Platform == '__WXMSW__':
+            acc_init =[
+                (wx.ACCEL_CTRL, ord('C'), wb_ids.id_SP_EditCopy),
+                (wx.ACCEL_CTRL, ord('X'), wb_ids.id_SP_EditCut),
+                (wx.ACCEL_CTRL, ord('V'), wb_ids.id_SP_EditPaste),
+
+                (wx.ACCEL_CTRL, ord('A'), wb_ids.id_SP_Add),
+                (wx.ACCEL_CTRL, ord('L'), wb_ids.id_SP_History),
+                (wx.ACCEL_CTRL, ord('I'), wb_ids.id_SP_Info),
+                (wx.ACCEL_CTRL, ord('P'), wb_ids.id_SP_Properties),
+                (wx.ACCEL_CTRL, ord('R'), wb_ids.id_SP_Revert),
+                (wx.ACCEL_CTRL, ord('U'), wb_ids.id_SP_Update),
+                (wx.ACCEL_NORMAL, wx.WXK_DELETE, wb_ids.id_SP_Delete),
+                (wx.ACCEL_NORMAL, wx.WXK_RETURN, wb_ids.id_Return_Hotkey),
+                ]
+            acc_init.append( (wx.ACCEL_CTRL, ord('O'), wb_ids.id_Shell_Open) )
+        else:
+            # Unix
+            acc_init =[
+                (wx.ACCEL_CTRL, ord('C'), wb_ids.id_SP_EditCopy),
+                (wx.ACCEL_CTRL, ord('X'), wb_ids.id_SP_EditCut),
+                (wx.ACCEL_CTRL, ord('V'), wb_ids.id_SP_EditPaste),
+
+                (wx.ACCEL_CTRL, ord('A'), wb_ids.id_SP_Add),
+                (wx.ACCEL_CTRL, ord('L'), wb_ids.id_SP_History),
+                (wx.ACCEL_CTRL, ord('I'), wb_ids.id_SP_Info),
+                (wx.ACCEL_CTRL, ord('P'), wb_ids.id_SP_Properties),
+                (wx.ACCEL_CTRL, ord('R'), wb_ids.id_SP_Revert),
+                (wx.ACCEL_CTRL, ord('U'), wb_ids.id_SP_Update),
+                (wx.ACCEL_NORMAL, wx.WXK_DELETE, wb_ids.id_SP_Delete),
+                (wx.ACCEL_NORMAL, wx.WXK_RETURN, wb_ids.id_Return_Hotkey),
+                ]
+
+        acc_tab = wx.AcceleratorTable( acc_init )
+        self.tree_ctrl.SetAcceleratorTable( acc_tab )
+
         self.tree_drop_target = SvnDropTarget( self )
         self.tree_ctrl.SetDropTarget( self.tree_drop_target )
 
         wx.EVT_SIZE( self, try_wrapper( self.OnSize ) )
         wx.EVT_TREE_ITEM_EXPANDING( self, self.tree_ctrl.getId(), try_wrapper( self.OnItemExpanding ) )
         wx.EVT_TREE_SEL_CHANGED( self, self.tree_ctrl.GetId(), try_wrapper( self.OnTreeSelChanged ) )
+        wx.EVT_LEFT_DOWN( self.tree_ctrl, try_wrapper( self.OnLeftDown ) )
         wx.EVT_RIGHT_DOWN( self.tree_ctrl, try_wrapper( self.OnRightDown ) )
         wx.EVT_RIGHT_UP( self.tree_ctrl, try_wrapper( self.OnRightUp ) )
 
@@ -75,19 +133,24 @@ class WbTreePanel(wx.Panel):
             wx.EVT_TREE_END_DRAG( self.tree_ctrl, self.tree_ctrl.GetId(), self.app.eventWrapper( self.OnDragEnd ) )
 
         wx.EVT_SET_FOCUS( self.tree_ctrl, self.OnSetFocus )
-        wx.EVT_KILL_FOCUS( self.tree_ctrl, self.OnKillFocus )  
+        wx.EVT_KILL_FOCUS( self.tree_ctrl, self.OnKillFocus )
 
         # start up by skipping ui updates until we have the tree control initialised
-        self._skip_update_ui = True
+        self.__skip_update_ui = True
+
+    def __repr__( self ):
+        return '<WbTreePanel %r>' % self.getSelectionProjectInfo()
 
     def initFrame( self ):
         self.initTree( self.app.prefs.getProjects().getProjectList() )
         self.gotoBookmark( self.last_position_bookmark_name )
 
     def OnSetFocus( self, event ):
+        if wb_config.debug_selection: print 'Z: WbTreePanel OnSetFocus'
         self.frame.setEventHandler( self )
 
     def OnKillFocus( self, event ):
+        if wb_config.debug_selection: print 'Z: WbTreePanel OnKillFocus'
         #self.frame.clearEventHandler()
         pass
 
@@ -99,16 +162,34 @@ class WbTreePanel(wx.Panel):
         #print 'WbTreePanel.OnDragEnd'
         pass
 
-    def getSelectionProjectInfo( self ):
+    def getSelectionProjectHandler( self ):
         item = self.tree_ctrl.GetSelection()
         if not item:
             return None
 
-        tree_pi = self.tree_ctrl.GetPyData( item ).getProjectInfo()
-        if not tree_pi:
+        handler = self.tree_ctrl.GetPyData( item )
+        if not handler:
             return None
 
-        return tree_pi
+        return handler
+
+    def getSelectionProjectInfo( self ):
+        handler = self.getSelectionProjectHandler()
+        if not handler:
+            return None
+
+        return handler.getProjectInfo()
+
+    def getProjectTopProjectInfo( self ):
+        item = self.tree_ctrl.GetSelection()
+
+        while item:
+            handler = self.tree_ctrl.GetPyData( item )
+            if handler.isProjectParent():
+                return handler.getProjectInfo()
+            item = self.tree_ctrl.GetItemParent( item )
+
+        return None
 
     def savePreferences( self ):
         bm_prefs = self.app.prefs.getBookmarks()
@@ -117,10 +198,10 @@ class WbTreePanel(wx.Panel):
 
         tree_pi = self.getSelectionProjectInfo()
         if tree_pi is not None:
-            bm_prefs.addBookmark( self.last_position_bookmark_name, tree_pi )
+            bm_prefs.addBookmark( tree_pi, name=self.last_position_bookmark_name )
 
     def initTree( self, project_info_list ):
-        self._skip_update_ui = True
+        self.__skip_update_ui = True
 
         self.root_item = self.tree_ctrl.AddRoot("Projects:")
         self.tree_ctrl.SetPyData( self.root_item, RootTreeItem() )
@@ -140,15 +221,15 @@ class WbTreePanel(wx.Panel):
             self.tree_ctrl.SetItemHasChildren( item, handler.mayExpand() )
             self.tree_ctrl.SetItemTextColour( item, handler.getTreeNodeColour() )
 
-        self.tree_ctrl.Expand( self.root_item )
+        self.tree_ctrl.SortChildren( self.root_item )
+        #self.tree_ctrl.Expand( self.root_item )
 
-        self._skip_update_ui = False
+        self.__skip_update_ui = False
 
         if first_item:
             self.tree_ctrl.SelectItem( first_item )
             self.tree_ctrl.EnsureVisible( first_item )
 
-
     def updateTreeSelectedItem( self ):
         self.updateTreeItem( self.tree_ctrl.GetSelection() )
 
@@ -194,6 +275,7 @@ class WbTreePanel(wx.Panel):
                 self.tree_ctrl.SetPyData( child_item, child_handler )
                 self.tree_ctrl.SetItemTextColour( child_item, wx.BLUE )
 
+        self.tree_ctrl.SortChildren( this_item )
 
         child_item, cookie = self.tree_ctrl.GetFirstChild( this_item )
         while child_item:
@@ -312,14 +394,18 @@ class WbTreePanel(wx.Panel):
 
     # Handler for when a new selection is made in the tree control child
     def OnTreeSelChanged( self, event ):
-        if self._skip_update_ui:
+        if wb_config.debug_selection: print 'Z: WbTreePanel OnTreeSelChanged __skip_update_ui',self.__skip_update_ui
+        if self.__skip_update_ui:
             return
 
+        tree_item = self.getItemHandler( event )
+        self.changedSelection( tree_item )
+
+    def changedSelection( self, tree_item ):
+        if wb_config.debug_selection: print 'Z: WbTreePanel changedSelection'
         self.frame.clearUpdateUiState()
         self.frame.setEventHandler( self )
 
-        tree_item = self.getItemHandler( event )
-
         # Update the contents of the list control to reflect the new selection
         if not tree_item:
             return
@@ -366,12 +452,6 @@ class WbTreePanel(wx.Panel):
         handler.getProjectInfo().updateStatus()
         self.updateTreeItem( this_item )
 
-    def OnRightDown( self, event ):
-        # move the selection to the clicked on node
-        point = event.GetPosition();
-        item, flags = self.tree_ctrl.HitTest( point )
-        self.tree_ctrl.SelectItem( item )
-
     def isTreeHandler( self ):
         return True
 
@@ -385,31 +465,59 @@ class WbTreePanel(wx.Panel):
 
         handler = self.tree_ctrl.GetPyData( item )
         if not handler:
-            return
+            return None
 
         return handler.getState()
 
+    def OnLeftDown( self, event ):
+        if wb_config.debug_selection: print 'OnLeftDown',
+        item = self.tree_ctrl.GetSelection()
+        if not item:
+            if wb_config.debug_selection: print 'no item selected'
+            return
+
+        handler = self.tree_ctrl.GetPyData( item )
+        if not handler:
+            if wb_config.debug_selection: print 'no handler'
+            return
+        if wb_config.debug_selection: print 'handler %r' % handler
+        event.Skip()
+
+    def OnRightDown( self, event ):
+        # move the selection to the clicked on node
+        point = event.GetPosition();
+        item, flags = self.tree_ctrl.HitTest( point )
+        self.tree_ctrl.SelectItem( item )
+        self.changedSelection( self.tree_ctrl.GetPyData( item ) )
+
+        if wx.Platform == '__WXMAC__':
+            self.popUpContextMenu( item, point )
+
     def OnRightUp( self, event ):
         # move the selection to the clicked on node
         point = event.GetPosition();
         item, flags = self.tree_ctrl.HitTest( point )
         self.tree_ctrl.SelectItem( item )
+        self.changedSelection( self.tree_ctrl.GetPyData( item ) )
 
+        if wx.Platform != '__WXMAC__':
+            self.popUpContextMenu( item, point )
+
+    def popUpContextMenu( self, item, point ):
         handler = self.tree_ctrl.GetPyData( item )
         if not handler:
             return None
 
         self.frame.getUpdateUiState()
 
-        if True:
-            menu = handler.getContextMenu( self.frame.ui_state_tree )
-            if handler.isProjectParent():
-                menu.AppendSeparator()
-                menu.Append( wb_ids.id_Project_Update, 'Update Project' )
-                menu.Append( wb_ids.id_Project_Delete, 'Delete Project' )
+        menu = handler.getContextMenu( self.frame.ui_state_tree )
+        if handler.isProjectParent():
+            menu.AppendSeparator()
+            menu.Append( wb_ids.id_Project_Update, 'Update Project' )
+            menu.Append( wb_ids.id_Project_Delete, 'Delete Project' )
 
-            self.tree_ctrl.PopupMenu( menu, event.GetPosition() )
-            menu.Destroy()
+        self.tree_ctrl.PopupMenu( menu, point )
+        menu.Destroy()
 
 
     def OnProjectAdd( self, event ):
@@ -434,6 +542,7 @@ class WbTreePanel(wx.Panel):
             else:
                 self.tree_ctrl.SetItemHasChildren( item, False )
 
+            self.tree_ctrl.SortChildren( item )
             self.tree_ctrl.SelectItem( item )
             self.tree_ctrl.EnsureVisible( item )
 
@@ -497,6 +606,7 @@ class WbTreePanel(wx.Panel):
             else:
                 self.tree_ctrl.SetItemHasChildren( item, False )
 
+            self.tree_ctrl.SortChildren( self.root_item )
             self.tree_ctrl.SelectItem( item )
 
     # command handlers
@@ -522,6 +632,16 @@ class WbTreePanel(wx.Panel):
 
         wb_shell_commands.FileBrowser( self.app, handler.getProjectInfo() )
 
+    def OnSpEditCopy( self ):
+        return self.Sp_Dispatch( 'Cmd_Dir_EditCopy' )
+
+    def OnSpEditCut( self ):
+        return self.Sp_Dispatch( 'Cmd_Dir_EditCut' )
+
+    def OnSpEditPaste( self ):
+        return self.Sp_Dispatch( 'Cmd_Dir_EditPaste' )
+
+
     def OnSpAdd( self ):
         return self.Sp_Dispatch( 'Cmd_Dir_Add' )
 
@@ -546,24 +666,41 @@ class WbTreePanel(wx.Panel):
     def OnSpMkdir( self ):
         return self.Sp_Dispatch( 'Cmd_Dir_Mkdir' )
 
+    def OnSpNewFile( self ):
+        return self.Sp_Dispatch( 'Cmd_Dir_NewFile' )
+
     def OnSpProperties( self ):
         return self.Sp_Dispatch( 'Cmd_Dir_Properties' )
 
     def OnSpRename( self ):
         return self.Sp_Dispatch( 'Cmd_Dir_Rename' )
 
+    def OnReportUpdates( self ):
+        return self.Sp_Dispatch( 'Cmd_Dir_ReportUpdates' )
+
+    def OnReportLocksWc( self ):
+        return self.Sp_Dispatch( 'Cmd_Dir_ReportLocksWc' )
+
+    def OnReportLocksRepos( self ):
+        return self.Sp_Dispatch( 'Cmd_Dir_ReportLocksRepos' )
+
+    def OnReportBranchChanges( self ):
+        return self.Sp_Dispatch( 'Cmd_Dir_ReportBranchChanges' )
+
     def OnSpRevert( self ):
         return self.Sp_Dispatch( 'Cmd_Dir_Revert' )
 
     def OnSpUpdate( self ):
         return self.Sp_Dispatch( 'Cmd_Dir_Update' )
 
+    #----------------------------------------
     def OnSpCopy( self, filename_list ):
         return self.Sp_DispatchDrop( 'Cmd_Dir_Copy', filename_list )
 
     def OnSpMove( self, filename_list ):
         return self.Sp_DispatchDrop( 'Cmd_Dir_Move', filename_list )
 
+    #----------------------------------------
     def Sp_Dispatch( self, sp_func_name ):
         self.app.trace.info( 'WbTreePanel.Sp_Dispatch( %s ) event' % sp_func_name )
         item = self.tree_ctrl.GetSelection()
@@ -575,7 +712,12 @@ class WbTreePanel(wx.Panel):
             return None
 
         self.app.trace.info( 'WbTreePanel.Sp_Dispatch( %s ) calling' % sp_func_name )
-        return getattr( handler, sp_func_name )()    
+        fn = getattr( handler, sp_func_name, None )
+        if fn is None:
+            print 'Not implemented',sp_func_name
+            return None
+        else:
+            return fn()    
 
     def Sp_DispatchDrop( self, sp_func_name, filename_list ):
         self.app.trace.info( 'WbTreePanel.Sp_DispatchDrop( %s )' % sp_func_name )
@@ -671,15 +813,19 @@ class SvnDropTarget(wx.PyDropTarget):
 #
 #--------------------------------------------------------------------------------
 class WbTreeCtrl(wx.TreeCtrl):
-    ''' TreeCtrlEx '''
-
     def __init__( self, parent, app ):
         self.app = app
         self.id_tree = wx.NewId()
+
+        if wx.Platform == '__WXMSW__':
+            style = wx.TR_HAS_BUTTONS|wx.NO_BORDER
+        else:
+            style = wx.TR_HAS_BUTTONS|wx.NO_BORDER|wx.TR_HIDE_ROOT
+ 
         wx.TreeCtrl.__init__( self, parent, self.id_tree, 
                 wx.DefaultPosition,
                 wx.DefaultSize,
-                wx.TR_HAS_BUTTONS|wx.NO_BORDER )
+                style )
 
         self.item_last_used = None
 
